package anune;

import java.io.Serializable;
import java.util.ArrayList;


public class Layer implements Serializable {
	
	
	public final ArrayList<Neuron> neurons;
	protected float[] output;
	private float[] outputBuffer;
	private final Layer prevLayer;
	
	
	public Layer(int layerSize, Layer prevLayer) {
		this.prevLayer = prevLayer;
		this.neurons = new ArrayList<>(layerSize);
		this.output = new float[layerSize];
		this.outputBuffer = new float[layerSize];
		
		int prevLayerSize = prevLayer == null ? 0 : prevLayer.neurons.size();
		for (int i = 0; i < layerSize; i++) {
			if (this instanceof InputLayer){
				neurons.add(new Neuron(prevLayerSize, 0));
			}
			else neurons.add(new Neuron(prevLayerSize, layerSize));
		}
		
	}
	
	
	public Layer(int layerSize) {
		this(layerSize, null);
	}
	
	
	public void calcOutput() {
		for (int i = 0; i < output.length; i++) {
			outputBuffer[i] = neurons.get(i).activate(prevLayer.output, output);
		}
		
		// Copy from outputBuffer to output
		System.arraycopy(outputBuffer, 0, output, 0, output.length);
	}
	
	
	public float[] getOutput() {
		return this.output;
	}
}
