package anune;

import java.io.Serializable;
import anune.util.Utils;


public class Neuron implements Serializable {
	
	public final float weights[];
	public final float sameLayerWeights[];
	public float tau;
	public float gain;
	public float bias;
	private float y;

	
	public Neuron(int numInputs, int layerSize) {
		weights = new float[numInputs];
		sameLayerWeights = new float[layerSize];
		
		setRandomWeights();
		
		gain = 1.0f;
		tau  = 1.0f;
		bias = 0.0f;
		y    = 0.0f;
	}
	
	
	public void setRandomWeights() {
		for (int i = 0; i < weights.length; i++)
			weights[i] = Utils.randomFloat()*2 - 1;
		
		for (int i = 0; i < sameLayerWeights.length; i++) {
			sameLayerWeights[i] = Utils.randomFloat()*2 -1;
		}
	}
	
	
	public float activate(float[] inputs, float[] sameLayerInputs) {
		assert inputs.length == weights.length;
		assert sameLayerInputs.length == sameLayerWeights.length;
		
		// Set sum = 0.0 + bias.
		float sum = bias;
		
		// Calculate activation from inputs.
		for (int i = 0; i < inputs.length; i++)
			sum += inputs[i] * weights[i];
		
		// Calculate activation from same layer inputs.
		for (int i = 0; i < sameLayerInputs.length; i++)
			sum += sameLayerInputs[i] * sameLayerWeights[i];
		
		// Update neuron activation.
		y += (sum - y) / tau;
		
		// Anti-infinity check fette retard java
		if (y < -1e20f) y = -1e20f;
		else if (y > 1e20f) y = 1e20f;
		
		// Run activation function. TODO: Add more activation functions?
		return sigmoid(y, gain);
	}
	
	
	/**
	 * Returns the weights for this neuron in a new array.
	 * @return 
	 */
	public float[] getWeights() {
		float[] w = new float[weights.length + sameLayerWeights.length];
		getWeights(w);
		return w;
	}
	
	
	/**
	 * Places the weights for this neuron in the provided array.<br>
	 * Passed array must have the appropriate length.
	 * @param w The array to place the weights.
	 */
	public void getWeights(float[] w) {
		if (w.length != weights.length + sameLayerWeights.length)
			throw new IllegalArgumentException(
					String.format("Provided weight array has length %s, expected %d.", w.length, weights.length + sameLayerWeights.length));
		
		System.arraycopy(weights, 0, w, 0, weights.length);
		System.arraycopy(sameLayerWeights, 0, w, weights.length, sameLayerWeights.length);
	}
	
	
	/**
	 * Set all weights.
	 * @param w 
	 */
	public void setWeights(double[] w) {
		if (w.length != weights.length + sameLayerWeights.length)
			throw new IllegalArgumentException(
					String.format("Provided weight array has length %s, expected %d.", w.length, weights.length + sameLayerWeights.length));
		
		System.arraycopy(w, 0, weights, 0, weights.length);
		System.arraycopy(w, weights.length, sameLayerWeights, 0, sameLayerWeights.length);
	}
	
	
	private static float sigmoid(float y, float g) {
		float v = 1.0f / (1.0f + (float)Math.exp(-g*y));
		return v;
	}
	
}
