package creativebehaviour.ea;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.models.BehaviourGenotype;
import creativebehaviour.models.physical.TreeGenotype;
import eval.ea.EA;
import eval.util.Config;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public abstract class BrainEA extends EA {
	
	protected TreeGenotype currentTree;
	
	
	public BrainEA(Config cfg) {
		super(cfg);
	}
	
	
	public void setCurrentTree(TreeGenotype tg) {
		this.currentTree = tg;
	}
	
	
	@Override
	public boolean evolve() {
		boolean result = super.evolve();
		this.currentTree.setBehaviourGenotype((BehaviourGenotype) this.getWinner().getGenotype());
		return result;
	}
	
	
	public enum Method {
		CTRNN, FUZZY_LOGIC, NEAT;
	}
	
	
	public static BrainEA createBrainEA(Method method) {
		switch (method) {
			case CTRNN:
				return new NetworkWeightsEA(CreativeBehaviour.weightsCfg);
				
			case FUZZY_LOGIC:
				return new FuzzyEA(CreativeBehaviour.fuzzyCfg);
				
			case NEAT:
				return new NeatEA(CreativeBehaviour.neatCfg);
				
			default:
				throw new AssertionError(method.name());
		}
	}
}
