package creativebehaviour.ea;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.models.physical.EntityPhenotype;
import creativebehaviour.models.physical.TreeGenotype;
import creativebehaviour.util.tree.Node;
import creativebehaviour.util.Utils;
import creativebehaviour.util.profiling.Timer;
import eval.ea.EA;
import eval.ea.OnChangeListener;
import eval.ea.OnChangedEvent;
import eval.models.Genotype;
import eval.models.Phenotype;
import eval.models.Population;
import eval.util.Config;
import java.io.IOException;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class CreatureEvolveEA extends EA {

	/** The number of entities to attach to initial creatures. */
	private static final int NUM_START_ENTS = CreativeBehaviour.crebeCfg.getInt("num_start_entities");
	
	
	public CreatureEvolveEA(Config cfg) throws IOException {
		super(cfg);
		
		Timer t = new Timer();
		
		addListener(new OnChangeListener() {
			@Override
			public void handle(OnChangedEvent e) {
				if (e == OnChangedEvent.NEW_GENERATION) {
					System.out.println("\nNEW CREATURE EVOLVE GENERATION");
					System.out.println(t);
					t.reset();
				}
			}
		});
	}
	
	
	@Override
	public Population generateInitalPopulation(int size) {
		Population pop = new Population(size);
		
		for (int i = 0; i < size; i++) {
			TreeGenotype treeg = new TreeGenotype();
			
			int entAdded = 0;
			while (entAdded < NUM_START_ENTS) {
				for (Node<Wrapper> node : treeg.tree.nodeIterator()) {
					if (node.obj == null && Utils.randomEvent(0.1)) { // TODO: hardcoded config
						entAdded++;
						
						// Select a random entity, excluding body.
						EntType t;
						do {
							t = EntType.values()[Utils.randomInt(EntType.ENT_COUNT)];
						} while (t == EntType.BODY);
						
						Wrapper w = t.createWrapper();
						treeg.addEntityWrapperToNode(node, w);
					}
				}
			}
			
			pop.add(new EntityPhenotype(treeg));
		}
		
		return pop;
	}
	

	@Override
	public Phenotype generatePhenotype(Genotype g) {
		return new EntityPhenotype((TreeGenotype) g);
	}
	
}
