package creativebehaviour.ea;

import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.models.physical.TreeGenotype;
import creativebehaviour.models.neat.NeatGenotype;
import creativebehaviour.models.neat.NeatCreature;
import eval.ea.EA;
import eval.ea.OnChangedEvent;
import eval.models.Genotype;
import eval.models.Phenotype;
import eval.models.Population;
import eval.util.Config;
import eval.util.Pair;
import eval.util.Utils;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Endre
 */
public class NeatEA extends BrainEA {
	
	public static final int DIST_THRESH = 3; // TODO kanskje flytte over til config
	
	/** Class for assigning population into species **/
	private class Species {
		
		/** Identifier for the species **/
		//public int id;
		
		/** The cumulative adjusted fitness of the species **/
		public float cumAdjustedFitness = 0;
		public int offspringCount = 0;
		
		/** The representative genotype for this species **/
		public NeatGenotype genotypeRep;
		
		/** Individuals in the species, initial size 0 **/
		public Population pop;
		
		public Species(int populationSize){
			pop = new Population(populationSize);
		}
		
	}
	
	private ArrayList<Species> speciesList;
	
	public NeatEA(Config config, TreeGenotype currentTree) {
		super(config);
		this.currentTree = currentTree;
		this.speciesList = new ArrayList<>();
		
	}
	
	public NeatEA(Config config) {
		super(config);
		this.currentTree = new TreeGenotype();
		this.speciesList = new ArrayList<>();
	}
	
	/**
	 * 
	 * @return true if max fitness is reached
	 */
	@Override
	public boolean evolve() {
		printVerbose("Starting evolution with " + threads + " thread(s).");
		
		reset();
		running = true;
		
		// Initialization
		population = generateInitalPopulation(populationSize);
		calculateFitnesses(population);
		notifyListeners(OnChangedEvent.FITNESS_CALCULATED);
		this.speciesList.clear();
		this.speciesList.add(new Species(populationSize));
		this.speciesList.get(0).genotypeRep = (NeatGenotype) population.get(0).getGenotype();
		this.speciesList.get(0).pop.add(population.get(0));
		
		for (int i = 1; i < population.getCount(); i++) {
			Phenotype n = population.get(i);
			NeatGenotype ng = (NeatGenotype) n.getGenotype();
			
			boolean addedToSpecies = false;
			for (Species s : this.speciesList) {
				double dist = ng.distanceTo(s.genotypeRep);
				if (dist < DIST_THRESH) {
					s.pop.add(n);
					addedToSpecies = true;
					break;
				}
			}
			if (!addedToSpecies) {
				Species s = new Species(populationSize);
				s.genotypeRep = ng;
				s.pop.add(n);
				this.speciesList.add(s);
			}
		}
		double prevFitness = 0;
		int noImprovementCount = 0;
		while (true) {
			brokerindex = 0;
			generation++;
		
			population.sort(false);
			
			// Fitness data and stats retreival
			genwinner = population.getMaxFitness();
			notifyListeners(OnChangedEvent.NEW_GENERATION);
			
			// Check paused status
			while (paused) {
				try {
					synchronized (pauseObj) {
						pauseObj.wait();
					}
				} catch (InterruptedException e) {
				}
			}
			
			if (winner == null || (genwinner.getFitness() > winner.getFitness())) {
				winner = genwinner;
				notifyListeners(OnChangedEvent.NEW_BEST_FITNESS);
			}
			avgfitness = population.getAvgFitness();
			stdfitness = population.getStdFitness();
			bestfitness = genwinner.getFitness();
			
			fitnessAverages.add(avgfitness);
			fitnessBests.add(bestfitness);
			fitnessDeviations.add(stdfitness);
			
			
			
			// Print info
			if (!quiet) {
				if (verbose) {
					double progress = (generation / (double)generationLimit) * 100.0;
					System.out.println();
					System.out.printf("G# %d     max_f: %.4f     avg_f: %.4f     SD: %.4f     Progress: %.2f%%\n", generation, bestfitness, avgfitness, stdfitness, progress);
					System.out.println("Generation winner: "+ genwinner);
				} else {
					StringBuilder sb = new StringBuilder();
					sb.append("\r");
					Utils.buildProgressbar(sb, 50, generation, generationLimit);
					sb.append("  Gen: ");
					sb.append(generation);
					System.out.print(sb);
					System.out.flush();
				}
			}
			
			if (isDone()) break;
			if (bestfitness == prevFitness) noImprovementCount++;
			else noImprovementCount = 0;
			
			if (noImprovementCount >= 20) {
				// reproduce with only top 2
				population.sort();
				Genotype p1 = population.get(0).getGenotype();
				Genotype p2;
				if (population.getCount() == 1) {
					p2 = p1;
				} 
				else {
					p2 = population.get(1).getGenotype();
				}
				offspring.clear();
				while (offspring.getCount() != populationSize) {
					Genotype offspr;
					if (Utils.randomEvent(crossoverrate)) {
						offspr = p1.crossover(p2, crossoverType, 1); // Best parent is always p1
					} else {
						offspr = p1.copy();
					}

					offspr.mutate(mutationrate);

					Phenotype child = generatePhenotype(offspr);
					offspring.add(child);
				}
				
				noImprovementCount = 0;
			}
			else {
				float totCumAdjustedFitness = 0;
				for (Species s : this.speciesList) {
					for (Phenotype p : s.pop) {
						s.cumAdjustedFitness += Math.max(p.getFitness(),0)/(double)s.pop.getCount();
					}
					totCumAdjustedFitness += s.cumAdjustedFitness;
				}
				// Find offspring per species
				int totalOffspring = 0;
				for (Species s : this.speciesList) {
					float percentage = s.cumAdjustedFitness / totCumAdjustedFitness;
					s.offspringCount = (int) Math.floor(populationSize*percentage);
					totalOffspring += s.offspringCount;
				}
				// Assure that the total offspring count is equal to the population size
				int deltaOffspring = populationSize - totalOffspring;
				while(deltaOffspring != 0) {
					if (deltaOffspring < 0) {
						int index = Utils.randomInt(this.speciesList.size());
						if (this.speciesList.get(index).offspringCount > 0) {
							this.speciesList.get(index).offspringCount--;
							deltaOffspring++;
						}
					}
					else {
						this.speciesList.get(Utils.randomInt(this.speciesList.size())).offspringCount++;
						deltaOffspring--;
					}
				}
				// Eliminate lowest performing members
				// Parent selection
				List<Pair<Phenotype>> parents = new ArrayList<>();

				for (Species s : this.speciesList) {
					parents.addAll(parentSelection.selectParents(s.pop, s.offspringCount));
				}

				// Reproduction w/ crossover + mutation
				offspring.clear();
				int counter = 0;
				for (Pair<Phenotype> pair : parents) {
					Genotype p1 = pair.first.getGenotype();
					Genotype p2 = pair.second.getGenotype();
					int bestParent = 0;
					if (pair.first.getFitness() > pair.second.getFitness()) bestParent = 1;
					else if (pair.first.getFitness() < pair.second.getFitness()) bestParent = -1;
					Genotype offspr;
					if (Utils.randomEvent(crossoverrate)) {
						offspr = p1.crossover(p2, crossoverType, bestParent);
					} else {
						offspr = p1.copy();
					}

					offspr.mutate(mutationrate);
					
					Phenotype child = generatePhenotype(offspr);
					offspring.add(child);
					counter++;
				}
			}
			prevFitness = bestfitness;
			// Evaluate offspring
			calculateFitnesses(offspring);
			notifyListeners(OnChangedEvent.FITNESS_CALCULATED);
			
			// Specieate offspring and prepare next generation population
			population = offspring;
			this.speciesList.clear();
			this.speciesList.add(new Species(populationSize));
			this.speciesList.get(0).genotypeRep = (NeatGenotype) population.get(0).getGenotype();
			this.speciesList.get(0).pop.add(population.get(0));

			for (int i = 1; i < population.getCount(); i++) {
				Phenotype n =  population.get(i);
				NeatGenotype ng = (NeatGenotype) n.getGenotype();

				boolean addedToSpecies = false;
				for (Species s : this.speciesList) {
					if (ng.distanceTo(s.genotypeRep) < DIST_THRESH) {
						s.pop.add(n);
						addedToSpecies = true;
						break;
					}
				}
				if (!addedToSpecies) {
					Species s = new Species(populationSize);
					s.genotypeRep = ng;
					s.pop.add(n);
					this.speciesList.add(s);
				}
			}
			
			
			assert population.getCount() == populationSize;
		}
		
		running = false;
		return generation < generationLimit;
	}

	@Override
	public Population generateInitalPopulation(int size) {
		
		NeatGenotype.nextId = 0;
		NeatGenotype.nextNeuronId = 0;
		NeatGenotype.connMutations.clear();
		NeatGenotype.nodeMutations.clear();
		
		Population pop = new Population(size);
		NeatGenotype g;
//		if (currentTree.behaviourGenotype != null) {
//			g = (NeatGenotype) currentTree.behaviourGenotype;
//		}
//		else {
			int inputSize = 0;
			int outputSize = 0;
			for (Wrapper entwrapper : currentTree.tree) {
				if (entwrapper.isSensor()) inputSize += entwrapper.entType.getInputLenght();
				if (entwrapper.isActuator()) outputSize += entwrapper.entType.getOutputLenght();
			}
			g = new NeatGenotype(inputSize, outputSize);
			currentTree.setBehaviourGenotype(g);
//		}
		
		for (int i = 0; i < size; i++) {
			pop.add(new NeatCreature(new TreeGenotype(currentTree), new NeatGenotype(g)));
		}
		
		return pop;
	}

	@Override
	public Phenotype generatePhenotype(Genotype g) {
		return new NeatCreature(new TreeGenotype(currentTree), (NeatGenotype) g);
	}
	
}
