package creativebehaviour.ents;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.ents.misc.DebugColors;
import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.misc.InputEntity;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.graphics.Render;
import creativebehaviour.models.Creature;
import creativebehaviour.simulation.World;
import creativebehaviour.util.Pointf;
import creativebehaviour.util.Utils;
import eval.util.Config;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.geom.NoninvertibleTransformException;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Ear extends Entity implements InputEntity{

	
	public static class EarWrapper extends Wrapper {
		public float range = 300; // Pixels
		
		private static float range_mf;
		
		public EarWrapper() {
			super(EntType.EAR);
		}
		
		public static void parseMfConfig(Config cfg) {
			range_mf = cfg.getFloat("ear_range_mf");
		}

		@Override
		public void mutate(double mutateFactor) {
			range += mutateFactor * range_mf;
			if (range < 0) range = 0;
		}

		@Override
		public Entity createEntity() {
			return new Ear(this);
		}
		
		@Override
		public boolean isSensor() {
			return true;
		}
		
		@Override
		public boolean isActuator() {
			return false;
		}

		@Override
		public Wrapper deepCopy() {
			EarWrapper e = new EarWrapper();
			e.range = range;
			return e;
		}

		@Override
		public double calcUpkeep() {
			return range / 300.0;
		}

		@Override
		public EntType getEntType() {
			return EntType.EAR;
		}
	}
	
	public static final double BASE_UPKEEP = 1.0;

	private static final int LEFT_HEIGHT = 18;
	private static final int RIGHT_HEIGHT = 30;
	private static final int WIDTH = 14;
	private static final int INDENT = 6;
	
	public EarWrapper wrapper;
	private Polygon p = new Polygon();

	
	public Ear() {
		this(new EarWrapper());
	}
	
	
	public Ear(EarWrapper w) {
		super(EntType.EAR, BASE_UPKEEP);
		
		wrapper = w;
		
		p.addPoint(0, LEFT_HEIGHT/2);
		p.addPoint(WIDTH, RIGHT_HEIGHT/2);
		p.addPoint(WIDTH-INDENT, LEFT_HEIGHT/4);
		p.addPoint(WIDTH-INDENT, -LEFT_HEIGHT/4);
		p.addPoint(WIDTH, -RIGHT_HEIGHT/2);
		p.addPoint(0, -LEFT_HEIGHT/2);
	}
	
	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		Render.renderOutlinedShape(p, Color.BLACK, getColor(), Render.STANDARD_STROKE, g);
		
		if (CreativeBehaviour.DEBUG) {
			g.setColor(DebugColors.EAR_DEBUG_COLOR);
			g.setStroke(DebugColors.DEBUG_STROKE);
			g.drawOval((int)-wrapper.range, (int)-wrapper.range, (int)wrapper.range*2, (int)wrapper.range*2);
			g.setColor(Color.BLACK);
		}
		
		super.render(g);
	}
	
	
	@Override
	public double getInputValue(World world) {
		double soundLevel = 0;
		Pointf pos = this.getGlobalPos();
		
		for (Creature c : world.creatures) {
			if (c != getRootCreature()) {
				double dist = pos.distanceTo(c.position);
				if (dist < wrapper.range) {
					double thisSound = 1 - dist / wrapper.range;
					if (thisSound > soundLevel) soundLevel = thisSound;
				}
			}
		}
		
		return soundLevel;
	}
	
	
	@Override
	public double[] getInputData(World world) {
		return new double[] { getInputValue(world) };
	}
}
