package creativebehaviour.ents;

import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.graphics.Render;
import creativebehaviour.util.Pointf;
import java.awt.BasicStroke;
import java.awt.Color;
import creativebehaviour.util.Utils;
import eval.util.Config;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.NoninvertibleTransformException;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Fork extends Entity {
	
	public static class ForkWrapper extends Wrapper {
		public float fork_angle = 60; // Degrees
		public float arm_length = 40; // Pixels

		private static float fork_angle_mf;
		private static float arm_length_mf;
		
		public static void parseMfConfig(Config cfg) {
			fork_angle_mf = cfg.getFloat("fork_forkangle_mf");
			arm_length_mf = cfg.getFloat("fork_armlength_mf");
		}
		
		public ForkWrapper() {
			super(ATTACH_POINTS, EntType.FORK);
		}
		
		@Override
		public void mutate(double mutateFactor) {
			fork_angle += mutateFactor * fork_angle_mf;
			arm_length += mutateFactor * arm_length_mf;
			
			if (fork_angle < 5) fork_angle = 5;
			else if (fork_angle > 180) fork_angle = 180;
			if (arm_length < 0) arm_length = 0;
		}

		@Override
		public Entity createEntity() {
			return new Fork(this);
		}
		
		@Override
		public boolean isSensor() {
			return false;
		}
		
		@Override
		public boolean isActuator() {
			return false;
		}

		@Override
		public Wrapper deepCopy() {
			ForkWrapper f = new ForkWrapper();
			f.fork_angle = fork_angle;
			f.arm_length = arm_length;
			return f;
		}

		@Override
		public double calcUpkeep() {
			double sum = 0.0;
			sum += fork_angle / 60.0;
			sum += arm_length / 40.0;
			return sum / 20.0;
		}

		@Override
		public EntType getEntType() {
			return EntType.FORK;
		}
	}

	public static final double BASE_UPKEEP = 1.0;
	public static final int ATTACH_POINTS = 2;
	private static final BasicStroke FORK_THICKNESS_STROKE = new BasicStroke(12, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	
	public ForkWrapper wrapper;
	private Line2D lineShape;
	
	
	public Fork() {
		this(new ForkWrapper());
	}
	
	
	public Fork(ForkWrapper w) {
		super(EntType.FORK, BASE_UPKEEP);
		
		wrapper = w;
		lineShape = new Line2D.Double(0, 0, wrapper.arm_length, 0);
		
		addAttachPoint(0, 0, Math.toRadians(wrapper.fork_angle * 0.5));
		addAttachPoint(0, 0, Math.toRadians(-wrapper.fork_angle * 0.5));
		
		updateAttachPointOrigins();
	}

	
	public double getFork_angle() {
		return wrapper.fork_angle;
	}

	
	public void setFork_angle(double fork_angle) {
		wrapper.fork_angle = (float)fork_angle;
		updateAttachPointOrigins();
	}
	

	public double getArm_length() {
		return wrapper.arm_length;
	}
	

	public void setArm_length(double arm_length) {
		wrapper.arm_length = (float)arm_length;
		lineShape = new Line2D.Double(0, 0, wrapper.arm_length, 0);
		updateAttachPointOrigins();
	}
	
	
	public void setForkAttachment(Entity ent, int index) {
		assert attachPoints.size() == 2;
		if (index > 1) throw new IllegalArgumentException("Fork attachment index must be 0 or 1.");
		attachPoints.get(index).setChild(ent);
	}
	
	
	private void updateAttachPointOrigins() {
		assert attachPoints.size() == 2;
		
		Pointf p1 = attachPoints.get(0).parentRelativePos;
		Pointf p2 = attachPoints.get(1).parentRelativePos;
		
		double rad = Math.toRadians(wrapper.fork_angle * 0.5);
		p1.x = wrapper.arm_length * Math.cos(rad);
		p1.y = wrapper.arm_length * Math.sin(rad);
		
		p2.x = p1.x;
		p2.y = -p1.y;
	}
	
	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		resetTransform();
		
		transform.rotate(Math.toRadians(wrapper.fork_angle * 0.5));
		applyTransformation(g, transform);
		Render.renderOutlinedLine(lineShape, Color.BLACK, getColor(), Render.STANDARD_STROKE, FORK_THICKNESS_STROKE, g);
		revertTransformation(g, transform);
		
		attachPoints.get(0).render(g);
		
		applyTransformation(g, transform);
		Render.renderOutlinedLine(lineShape, Color.BLACK, getColor(), Render.STANDARD_STROKE, FORK_THICKNESS_STROKE, g);
		revertTransformation(g, transform);
		
		attachPoints.get(1).render(g);
		
		super.render(g);
	}
	
}
