package creativebehaviour.ents;

import creativebehaviour.ents.misc.OutputEntity;
import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.graphics.Render;
import creativebehaviour.util.Pointf;
import eval.util.Config;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Stroke;
import java.awt.geom.NoninvertibleTransformException;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Motor extends Entity implements OutputEntity {
	
	
	public static class MotorWrapper extends Wrapper {
		
		public float speed = 35; // pixels/sec?
		public float strength = 0.4f;

		private static float speed_mf;
		private static float strength_mf;
		
		public static void parseMfConfig(Config cfg) {
			speed_mf = cfg.getFloat("motor_speed_mf");
			strength_mf = cfg.getFloat("motor_strength_mf");
		}

		public MotorWrapper() {
			super(EntType.MOTOR);
		}
		
		@Override
		public void mutate(double mutateFactor) {
			speed += mutateFactor * speed_mf;
			strength += mutateFactor * strength_mf;
			
			if (speed < 0) speed = 0;
			if (strength < 0) strength = 0;
		}

		@Override
		public Entity createEntity() {
			return new Motor(this);
		}
		
		@Override
		public boolean isSensor() {
			return false;
		}
		
		@Override
		public boolean isActuator() {
			return true;
		}

		@Override
		public Wrapper deepCopy() {
			MotorWrapper m = new MotorWrapper();
			m.speed = speed;
			m.strength = strength;
			return m;
		}

		@Override
		public double calcUpkeep() {
			double sum = 0.0;
			sum += speed / 50.0;
			sum += strength / 0.5;
			return sum / 2.0;
		}

		@Override
		public EntType getEntType() {
			return EntType.MOTOR;
		}
	}
	

	public static final double BASE_UPKEEP = 1.0;
	
	private static final int LEFT_HEIGHT = 12;
	private static final int RIGHT_HEIGHT = 20;
	private static final int WIDTH = 14;
	
	private static final Stroke STROKE = new BasicStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	
	public MotorWrapper wrapper;
	private Polygon p;
	
	private transient double movingX1 = 0;
	private transient double movingX2 = 7;
	private transient double movingX3 = 14;
	
	
	public Motor() {
		this(new MotorWrapper());
	}
	
	
	public Motor(MotorWrapper w) {
		super(EntType.MOTOR, BASE_UPKEEP);
		
		wrapper = w;
		
		p = new Polygon();
		p.addPoint(0, -LEFT_HEIGHT/2);
		p.addPoint(WIDTH, -RIGHT_HEIGHT/2);
		p.addPoint(WIDTH, RIGHT_HEIGHT/2);
		p.addPoint(0, LEFT_HEIGHT/2);
	}
	
	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		g.setStroke(STROKE);
		g.setColor(Color.BLACK);
		
		movingX1 = (movingX1 + 0.5) % 21;
		movingX2 = (movingX2 + 0.5) % 21;
		movingX3 = (movingX3 + 0.5) % 21;
		
		int x = WIDTH - RIGHT_HEIGHT/2 - 16;
		g.drawArc(x + (int)movingX1, -(RIGHT_HEIGHT+ (x + (int)movingX1*2)/2)/2, RIGHT_HEIGHT+(int)movingX1, RIGHT_HEIGHT+ (x + (int)movingX1*2)/2, -45, 90);
		g.drawArc(x + (int)movingX2, -(RIGHT_HEIGHT+ (x + (int)movingX2*2)/2)/2, RIGHT_HEIGHT+(int)movingX2, RIGHT_HEIGHT+ (x + (int)movingX2*2)/2, -45, 90);
		g.drawArc(x + (int)movingX3, -(RIGHT_HEIGHT+ (x + (int)movingX3*2)/2)/2, RIGHT_HEIGHT+(int)movingX3, RIGHT_HEIGHT+ (x + (int)movingX3*2)/2, -45, 90);
		
		Render.renderOutlinedShape(p, Color.BLACK, Color.LIGHT_GRAY, Render.STANDARD_STROKE, g);
		super.render(g);
	}
	
	
	@Override
	public double performAction(double value) {
		double forceFactor = value * wrapper.strength;
		double angle = getGlobalRotation();
		Pointf force = new Pointf(Math.cos(angle), Math.sin(angle));
		force.normalize();
		force.scale(-forceFactor);
		getRootCreature().applyForce(force);
		return value;
	}
	
	
	@Override
	public double performActions(double[] activation) {
		return performAction(activation[0]);
	}
}

