package creativebehaviour.ents;

import creativebehaviour.ents.misc.OutputEntity;
import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.misc.InputEntity;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.graphics.Render;
import creativebehaviour.simulation.World;
import creativebehaviour.util.Utils;
import eval.util.Config;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.RoundRectangle2D;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Mouth extends Entity implements InputEntity {


	public static class MouthWrapper extends Wrapper {
		
		public static void parseMfConfig(Config cfg) {
		}

		public MouthWrapper() {
			super(EntType.MOUTH);
		}
		
		@Override
		public void mutate(double mutateFactor) {
		}
		
		@Override
		public boolean isActuator() {
			return false;
		}

		@Override
		public boolean isSensor() {
			return true;
		}

		@Override
		public Entity createEntity() {
			return new Mouth(this);
		}
		

		@Override
		public Wrapper deepCopy() {
			MouthWrapper m = new MouthWrapper();
			return m;
		}

		@Override
		public double calcUpkeep() {
			return 1;
		}

		@Override
		public EntType getEntType() {
			return EntType.MOUTH;
		}
	}

	public static final double BASE_UPKEEP = 1.0;
	private static final BasicStroke STROKE = new BasicStroke(1);
	
	private static final int HEIGHT = 40;
	private static final int WIDTH = 20;
	
	
	public MouthWrapper wrapper;
	private final Shape mouthShape;
	private boolean ateLastTick = false;
	
	
	public Mouth() {
		this(new MouthWrapper());
	}
	
	
	public Mouth(MouthWrapper w) {
		super(EntType.MOUTH, BASE_UPKEEP);
		
		wrapper = w;
		
		mouthShape = new RoundRectangle2D.Double(-WIDTH/2, -HEIGHT/2, WIDTH, HEIGHT, WIDTH, WIDTH);
	}
	
	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		Render.renderOutlinedShape(mouthShape, Color.BLACK, Color.WHITE, Render.STANDARD_STROKE, g);
		g.setStroke(STROKE);
		g.setColor(Color.BLACK);
		
		int TW = WIDTH / 4;
		g.drawLine(-WIDTH/2+TW, -HEIGHT/2, -WIDTH/2+TW, HEIGHT/2);
		g.drawLine(WIDTH/2-TW, -HEIGHT/2, WIDTH/2-TW, HEIGHT/2);
		
		final int NW = 4;
		for (int i = 0; i < NW; i++) {
			int y = (HEIGHT / (NW+1)) * (i + 1) - HEIGHT / 2;
			g.drawLine(-WIDTH/2, y, -WIDTH/2 + TW, y);
			g.drawLine(WIDTH/2, y, WIDTH/2 - TW, y);
		}
		
		g.setStroke(Render.STANDARD_STROKE);
		super.render(g);
	}
	
	
	@Override
	public double getInputValue(World world) {
		double v = ateLastTick ? 1 : 0;
		ateLastTick = false;
		return v;
	}
	
	
	@Override
	public double[] getInputData(World world) {
		return new double[] { getInputValue(world) };
	}
	
	
	public void setAteLateTick() {
		ateLastTick = true;
	}
}
