package creativebehaviour.ents;

import creativebehaviour.ents.misc.DebugColors;
import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.misc.InputEntity;
import creativebehaviour.CreativeBehaviour;
import static creativebehaviour.ents.misc.DebugColors.DEBUG_STROKE;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.graphics.Render;
import creativebehaviour.models.Creature;
import creativebehaviour.simulation.Food;
import creativebehaviour.simulation.FoodWorld;
import creativebehaviour.simulation.World;
import creativebehaviour.util.Pointf;
import creativebehaviour.util.Utils;
import eval.util.Config;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.RoundRectangle2D;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Nose extends Entity implements InputEntity {

	
	public static class NoseWrapper extends Wrapper {
		
		public float range = 200; // Pixels
		
		private static float range_mf;
		
		public static void parseMfConfig(Config cfg) {
			range_mf = cfg.getFloat("nose_range_mf");
		}

		public NoseWrapper() {
			super(EntType.NOSE);
		}

		@Override
		public void mutate(double mutateFactor) {
			range += mutateFactor * range_mf;
			if (range < 0) range = 0;
		}

		@Override
		public Entity createEntity() {
			return new Nose(this);
		}
		
		@Override
		public boolean isSensor() {
			return true;
		}
		
		@Override
		public boolean isActuator() {
			return false;
		}

		@Override
		public Wrapper deepCopy() {
			NoseWrapper n = new NoseWrapper();
			n.range = range;
			return n;
		}

		@Override
		public double calcUpkeep() {
			return range / 300.0;
		}

		@Override
		public EntType getEntType() {
			return EntType.NOSE;
		}

	}

	public static final double BASE_UPKEEP = 1.0;
	private static final int WIDTH = 16;
	private static final int HEIGHT = 32;
	private static final int NOSTRIL_RADIUS = 2;
	
	public NoseWrapper wrapper;
	private final Shape noseShape;
	private final Shape nostril1;
	private final Shape nostril2;
	
	
	public Nose() {
		this(new NoseWrapper());
	}
	
	
	public Nose(NoseWrapper w) {
		super(EntType.NOSE, BASE_UPKEEP);
		
		wrapper = w;
		
		noseShape = new RoundRectangle2D.Double(-WIDTH/2, -HEIGHT/2, WIDTH, HEIGHT, WIDTH, WIDTH);
		nostril1 = new Ellipse2D.Double(-NOSTRIL_RADIUS/2, -3*NOSTRIL_RADIUS, NOSTRIL_RADIUS*2, NOSTRIL_RADIUS*2);
		nostril2 = new Ellipse2D.Double(-NOSTRIL_RADIUS/2, 3*NOSTRIL_RADIUS, NOSTRIL_RADIUS*2, NOSTRIL_RADIUS*2);
	}
	
	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		Render.renderOutlinedShape(noseShape, Color.BLACK, getColor(), Render.STANDARD_STROKE, g);
		Render.renderOutlinedShape(nostril1, Color.BLACK, Color.WHITE, Render.STANDARD_STROKE, g);
		Render.renderOutlinedShape(nostril2, Color.BLACK, Color.WHITE, Render.STANDARD_STROKE, g);
		
		if (CreativeBehaviour.DEBUG) {
			g.setColor(DebugColors.NOSE_DEBUG_COLOR);
			g.setStroke(DEBUG_STROKE);
			g.drawOval(-(int)this.wrapper.range, -(int)this.wrapper.range, (int)this.wrapper.range*2, (int)this.wrapper.range*2);
			g.setColor(Color.BLACK);
		}
		
		super.render(g);
	}
	
	
	@Override
	public double getInputValue(World world) {
		double smell = 0.0;
		Pointf pos = this.getGlobalPos();
		
		// TODO: impl. creature smell
		
		if (world instanceof FoodWorld) {
			FoodWorld fworld = (FoodWorld)world;
			for (Food f : fworld.getFood()) {
				if (Math.abs(pos.distanceTo(f.position)) < wrapper.range) {
					if (f.nourishment >= 0) smell++;
					else smell--;
				}
			}
		}
		
		return smell;
	}
	
	
	@Override
	public double[] getInputData(World world) {
		return new double[] { getInputValue(world) };
	}
}
