package creativebehaviour.ents;

import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.graphics.Render;
import creativebehaviour.util.Pointf;
import creativebehaviour.util.Utils;
import eval.util.Config;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Spike extends Entity {
	
	public static class SpikeWrapper extends Wrapper{
		
		public float length = 20;
		public float damage = 0.5f;
		
		private static float length_mf;
		private static float damage_mf;
		
		public static void parseMfConfig(Config cfg) {
			length_mf = cfg.getFloat("spike_length_mf");
			damage_mf = cfg.getFloat("spike_damage_mf");
		}

		public SpikeWrapper() {
			super(EntType.SPIKE);
		}

		@Override
		public void mutate(double mutateFactor) {
			length += mutateFactor * length_mf;
			damage += mutateFactor * damage_mf;
			
			if (damage < 0) damage = 0;
			if (length < 5) length = 5;
		}

		@Override
		public Entity createEntity() {
			return new Spike(this);
		}
		
		@Override
		public boolean isSensor() {
			return false;
		}
		
		@Override
		public boolean isActuator() {
			return false;
		}

		@Override
		public Wrapper deepCopy() {
			SpikeWrapper s = new SpikeWrapper();
			s.length = length;
			s.damage = damage;
			return s;
		}

		@Override
		public double calcUpkeep() {
			double sum = 0.0;
			sum += length / 20.0;
			sum += damage / 0.5;
			return sum / 2.0;
		}

		@Override
		public EntType getEntType() {
			return EntType.SPIKE;
		}
	}

	public static final double BASE_UPKEEP = 1.0;
	
	public SpikeWrapper wrapper;
	private Polygon p = new Polygon();
	private Pointf speed = new Pointf();
	public Pointf prevPos = new Pointf();
	
	public Spike() {
		this(new SpikeWrapper());
	}
	
	
	public Spike(SpikeWrapper w) {
		super(EntType.SPIKE, BASE_UPKEEP);
		
		wrapper = w;
		
		p.addPoint(0, -5);
		p.addPoint((int) wrapper.length, 0);
		p.addPoint(0, 5);
		
	}
	
	
	public void setLength(double length) {
		wrapper.length = (float) length;
		p.xpoints[1] = (int) length;
	}
	
	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		Render.renderOutlinedShape(p, Color.BLACK, getColor(), Render.STANDARD_STROKE, g);
		super.render(g);
	}
	
	public Pointf getPointGlobalPos() {
		AffineTransform t = new AffineTransform();
		globalPosRecursive(t);
		t.translate(wrapper.length, 0);
		return new Pointf(t.getTranslateX(), t.getTranslateY());
	}
	
	public int getDamage() {
		return (int) (Math.pow(speed.length(), 1.7) * 0.001);
	}
	
	public void setSpeed(Pointf speed) {
		this.speed = speed;
	}
	
	public void setSpeed(int dx, int dy) {
		this.speed.x += dx;
		this.speed.y += dy;
	}
}
