package creativebehaviour.ents;

import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.misc.InputEntity;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.graphics.Render;
import creativebehaviour.models.Creature;
import creativebehaviour.simulation.World;
import creativebehaviour.util.Pointf;
import eval.util.Config;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.RoundRectangle2D;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class TouchSensor extends Entity implements InputEntity {
	
	
	public static class TouchSensorWrapper extends Wrapper {

		public static void parseMfConfig(Config cfg) {
		}

		public TouchSensorWrapper() {
			super(EntType.TOUCH_SENSOR);
		}
		
		@Override
		public void mutate(double mutateFactor) {
		}

		@Override
		public boolean isSensor() {
			return true;
		}

		@Override
		public boolean isActuator() {
			return false;
		}

		@Override
		public Entity createEntity() {
			return new TouchSensor(this);
		}

		@Override
		public Wrapper deepCopy() {
			TouchSensorWrapper t = new TouchSensorWrapper();
			return t;
		}

		@Override
		public double calcUpkeep() {
			return 1.0;
		}

		@Override
		public EntType getEntType() {
			return EntType.TOUCH_SENSOR;
		}
	}

	public static final double BASE_UPKEEP = 1.0;
	
	private static final BasicStroke STROKE = new BasicStroke(4);
	private static final Color BUTTON_COLOR = new Color(255, 80, 80);
	private static final int WIDTH = 16;
	private static final int HEIGHT = 16;
	
	public TouchSensorWrapper wrapper;
	private final Shape buttonShape;
	
	
	public TouchSensor() {
		this(new TouchSensorWrapper());
	}
	
	
	public TouchSensor(TouchSensorWrapper w) {
		super(EntType.TOUCH_SENSOR, BASE_UPKEEP);
		
		wrapper = w;
		
		buttonShape = new RoundRectangle2D.Double(0, -HEIGHT/4, WIDTH, HEIGHT/2, 0, 0);
	}

	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		Render.renderOutlinedShape(buttonShape, Color.BLACK, BUTTON_COLOR, Render.STANDARD_STROKE, g);
		g.setStroke(STROKE);
		g.setColor(Color.BLACK);
		g.drawLine(0, HEIGHT/2, 0, -HEIGHT/2);
		super.render(g);
	}
	
	
	@Override
	public double getInputValue(World world) {
		if (world.creatures.size() < 2) return 0;
		
		Pointf entPos = getTouchGlobalPos();
		for (Creature c : world.creatures) {
			if (c != getRootCreature()) {
				if (entPos.distanceTo(c.position) < c.rootEnt.getRadius()) return 1;
			}
		}
		
		return 0;
	}
	
	
	@Override
	public double[] getInputData(World world) {
		return new double[] { getInputValue(world) };
	}
	
	
	public Pointf getTouchGlobalPos() {
		AffineTransform t = new AffineTransform();
		globalPosRecursive(t);
		t.translate(WIDTH, 0);
		return new Pointf(t.getTranslateX(), t.getTranslateY());
	}
}
