package creativebehaviour.ents.misc;

import creativebehaviour.ents.Entity;
import creativebehaviour.util.Pointf;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.io.Serializable;
import java.util.Locale;


public class AttachPoint implements Serializable {

	public Entity parent;
	private Entity child;
	public Pointf parentRelativePos;
	private double rotation = 0;
	
	
	/**
	 * 
	 * @param parent
	 * @param x Parent relative pos x
	 * @param y Parent relative pos y
	 * @param child 
	 */
	public AttachPoint(Entity parent, double x, double y, Entity child) {
		this(parent, x, y, child, 0);
	}
	
	/**
	 * 
	 * @param parent
	 * @param x Parent relative pos x
	 * @param y Parent relative pos y
	 * @param child
	 * @param rotation
	 */
	public AttachPoint(Entity parent, double x, double y, Entity child, double rotation) {
		this.parent = parent;
		this.parentRelativePos = new Pointf(x, y);
		this.child = child;
		this.rotation = rotation;
	}
	
	
	/**
	 * 
	 * @param parent
	 * @param pos Parent relative position
	 * @param child 
	 */
	public AttachPoint(Entity parent, Pointf pos, Entity child) {
		this(parent, pos.x, pos.y, child);
	}
	
	
	public AttachPoint(Entity parent) {
		this(parent, 0, 0, null);
	}
	

	@Override
	public String toString() {
		return String.format(Locale.ENGLISH, "%s attached to %s at (%.1f, %.1f)",
				child == null ? "null" : child.getClass().getSimpleName(),
				parent == null ? "null" : parent.getClass().getSimpleName(),
				parentRelativePos.x,
				parentRelativePos.y
		);
	}
	
	
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		AffineTransform transform = new AffineTransform();
		transform.translate(parentRelativePos.x, parentRelativePos.y);
		transform.rotate(rotation);
		g.transform(transform);
		g.setColor(Color.BLACK);
		g.fillOval(-3, -3, 6, 6);
		if (child != null) child.render(g);
		transform.invert();
		g.transform(transform);
	}
	
	
	public void setChild(Entity ent) {
		this.child = ent;
		ent.attachedTo = this;
	}
	
	
	public Entity getChild() {
		return child;
	}
	
	
	public void setRotation(double rot) {
		this.rotation = rot;
	}
	
	
	public double getRotation() {
		return this.rotation;
	}
	
	
}
