package creativebehaviour.ents.misc;

import creativebehaviour.ents.Arm;
import creativebehaviour.ents.Body;
import creativebehaviour.ents.Ear;
import creativebehaviour.ents.Entity;
import creativebehaviour.ents.Eye;
import creativebehaviour.ents.Fork;
import creativebehaviour.ents.Motor;
import creativebehaviour.ents.Mouth;
import creativebehaviour.ents.Nose;
import creativebehaviour.ents.Spike;
import creativebehaviour.ents.TouchSensor;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.fuzzy.Variable;
import java.util.ArrayList;
import java.util.List;


public enum EntType {
	BODY         (Body.class),
	ARM          (Arm.class),
	EAR          (Ear.class),
	EYE          (Eye.class),
	FORK         (Fork.class),
	MOTOR        (Motor.class),
	MOUTH        (Mouth.class),
	NOSE         (Nose.class),
	SPIKE        (Spike.class),
	TOUCH_SENSOR (TouchSensor.class),
	;

	public static final int ENT_COUNT = EntType.values().length;
	private static boolean ioLegendsCreated = false;
	
	static {
		Variable.load();
	}
	
	public static void createIOLegends() {
		if (!ioLegendsCreated) {
			for (EntType t : values()) {
				List<Variable> vars = Variable.getVarsForEntity(t);
				ArrayList<Variable> inputVars = new ArrayList<>();
				ArrayList<Variable> outputVars = new ArrayList<>();
				
				for (Variable v : vars) {
					if (v.varType == Variable.Type.ACTION) outputVars.add(v);
					else if (v.varType == Variable.Type.SENSOR) inputVars.add(v);
				}
				
				t.inputLegend = inputVars.toArray(new Variable[inputVars.size()]);
				t.outputLegend = outputVars.toArray(new Variable[outputVars.size()]);
			}
			
			ioLegendsCreated = true;
		}
	}
	
	
	public final Class<? extends Entity> entClass;
	public Variable[] inputLegend;
	public Variable[] outputLegend;
	
	public int getInputLenght() {
		return inputLegend.length;
	}
	
	public int getOutputLenght() {
		return outputLegend.length;
	}
	
	public Wrapper createWrapper() {
		switch (this) {
			case BODY:
				return new Body.BodyWrapper();
			case ARM:
				return new Arm.ArmWrapper();
			case EAR:
				return new Ear.EarWrapper();
			case EYE:
				return new Eye.EyeWrapper();
			case FORK:
				return new Fork.ForkWrapper();
			case MOTOR:
				return new Motor.MotorWrapper();
			case MOUTH:
				return new Mouth.MouthWrapper();
			case NOSE:
				return new Nose.NoseWrapper();
			case SPIKE:
				return new Spike.SpikeWrapper();
			case TOUCH_SENSOR:
				return new TouchSensor.TouchSensorWrapper();
			default:
				throw new AssertionError(this.name());
		}
	}
	
	
	private EntType(Class<? extends Entity> clazz) {
		this.entClass = clazz;
	}
	
}
