package creativebehaviour.graphics;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.simulation.World;
import java.awt.HeadlessException;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.JFrame;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class MainFrame extends JFrame {

	private final RenderPanel renderPanel;
	private final Thread rthread;
	private boolean rthread_running = false;
	private int FPS = 30;
	
	
	public MainFrame(int width, int height, World world) throws HeadlessException {
		renderPanel = new RenderPanel(width, height, world);
		updateTitle();
		
		rthread = new Thread(() -> {
			long lastRender;
			
			while (rthread_running && isDisplayable()) {
				lastRender = System.nanoTime();
				renderPanel.tickWorld();
				renderPanel.render();
				updateTitle();
				
				try {
					long nanosToSleep = Math.max(0, 1_000_000_000 / FPS - System.nanoTime() + lastRender);
					Thread.sleep(nanosToSleep / 1_000_000);
				} catch (Exception e) {
					System.err.println(e);
				}
			}
		});
		
		
		
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		addKeyListener(new KeyListener() {
			@Override
			public void keyTyped(KeyEvent e) {
			}

			@Override
			public void keyPressed(KeyEvent e) {
				switch (e.getKeyCode()) {
					case KeyEvent.VK_SPACE:
						renderPanel.togglePause();
						break;
						
					case KeyEvent.VK_V:
						renderPanel.cycleFollow();
						break;
						
					case KeyEvent.VK_PLUS: case 107:
						FPS += 10;
						if (FPS > 200) FPS = 200;
						updateTitle();
						break;
						
					case KeyEvent.VK_MINUS: case 109:
						FPS -= 10;
						if (FPS < 10) FPS = 10;
						updateTitle();
						break;
						
					case KeyEvent.VK_D:
						CreativeBehaviour.DEBUG = !CreativeBehaviour.DEBUG;
						
					default:
						//System.out.println("Pressed key " + e.getKeyChar() + " " + e.getKeyCode());
				}
			}

			@Override
			public void keyReleased(KeyEvent e) {
			}
		});
		
		add(renderPanel);
		pack();
	}
	
	
	public void startRenderThread() {
		rthread_running = true;
		rthread.start();
	}
	
	
	public void render() {
		renderPanel.render();
	}
	
	
	public World getRenderWorld() {
		return renderPanel.world;
	}
	
	
	private void updateTitle() {
		setTitle("CreativeBehaviour  speed: " + FPS + "  tick: " + renderPanel.world.getTickCount());
	}
	
	
	public void cycleFollow() {
		renderPanel.cycleFollow();
	}
	
}
