package creativebehaviour.graphics;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Line2D;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Render {
	
	public static final Stroke STANDARD_STROKE = new BasicStroke(2);
	
	
	public static void renderOutlinedShape(Shape shape, Color outlineColor, Color fillColor, Stroke stroke, Graphics2D g) {
		Color oldColor = g.getColor();
		g.setColor(outlineColor);
		
		if (stroke != null) {
			g.fill(stroke.createStrokedShape(shape));
		}
		
		if (fillColor != null) {
			g.setColor(fillColor);
			g.fill(shape);
		}
		
		g.setColor(oldColor);
	}
	
	
	public static void renderOutlinedShape(Shape shape, Color outlineColor, Color fillColor, int strokeThickness, Graphics2D g) {
		Stroke stroke = new BasicStroke(strokeThickness, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		renderOutlinedShape(shape, outlineColor, fillColor, stroke, g);
	}
	
	
	public static void renderOutlinedLine(Line2D line, Color outlineColor, Color fillColor, Stroke stroke, Stroke lineShapeStroke, Graphics2D g) {
		Shape shape = lineShapeStroke.createStrokedShape(line);
		renderOutlinedShape(shape, outlineColor, fillColor, stroke, g);
	}
	
	
	public static void renderOutlinedLine(Line2D line, Color outlineColor, Color fillColor, Stroke stroke, int lineThickness, Graphics2D g) {
		Stroke shapeStroke = new BasicStroke(lineThickness, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		renderOutlinedLine(line, outlineColor, fillColor, stroke, shapeStroke, g);
	}
	
}
