package creativebehaviour.graphics.evolvedlistgui;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.graphics.MainFrame;
import static creativebehaviour.graphics.evolvedlistgui.EvolvedListGUI.FONT;
import creativebehaviour.models.Creature;
import creativebehaviour.util.Utils;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.util.Locale;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import creativebehaviour.util.io.CrebeIO;
import eval.ea.EA;
import eval.ea.OnChangeListener;
import eval.ea.OnChangedEvent;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JProgressBar;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class EvolvedListGUI extends JFrame {
	
	public static final Font FONT = new Font("Monospaced", Font.PLAIN, 12);
	
	private final JScrollPane scrollPane;
	private final JList list;
	private final DefaultListModel<CreatureListElement> listModel;
	private final JPanel controlContentPane;
	
	private final JFileChooser exportFileChooser;
	private final JFileChooser importFileChooser;
	
	private final DoubleInstrument maxFitnessInstrument;
	private final DoubleInstrument avgFitnessInstrument;
	private final DoubleInstrument stdFitnessInstrument;
	private final Instrument<Integer> genInstrument;
	
	private final JButton pauseButt;
	private final JButton resumeButt;
	private final JButton stopButt;
	private final JButton newEvoButt;
	private final JProgressBar genProgressBar;
	private final JProgressBar totalRunProgressBar;
	
	private EA currentEA;

	public EvolvedListGUI() throws HeadlessException {
		setTitle("Evovled creatures");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		listModel = new DefaultListModel<>();
		list = new JList<>(listModel);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		scrollPane = new JScrollPane(list);
		scrollPane.setPreferredSize(new Dimension(350, 700));
		
		setLayout(new GridLayout());
		
		list.addMouseListener(new MouseListener() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2 && list.getSelectedIndex() != 0) {
					CreatureListElement element = (CreatureListElement) list.getSelectedValue();
					if (element == null) return;
					
					Creature c = element.creature.copy();
					CreativeBehaviour.worldType = c.worldType;
					MainFrame frame = CreativeBehaviour.createMainFrame();
					frame.getRenderWorld().addCreature(c);
					frame.cycleFollow();
				}
			}

			@Override
			public void mousePressed(MouseEvent e) {
			}

			@Override
			public void mouseReleased(MouseEvent e) {
			}

			@Override
			public void mouseEntered(MouseEvent e) {
			}

			@Override
			public void mouseExited(MouseEvent e) {
			}
		});
		
		list.setFont(FONT);
		
		listModel.addElement(new CreatureListElement() {
			@Override
			public String toString() {
				return String.format("%-12s%-12s%-12s%s", "Hash code:", "Fitness:", "Upkeep:", "Imported:");
			}
		});
		
		add(scrollPane);
		
		JButton exportButt = new JButton("Export creature");
		exportButt.setEnabled(false);
		
		list.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				exportButt.setEnabled(list.getSelectedIndex() > 0);
			}
		});
		
		exportFileChooser = new JFileChooser(new File("."));
		exportFileChooser.setMultiSelectionEnabled(false);
		exportFileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
		exportFileChooser.setAcceptAllFileFilterUsed(false);
		exportFileChooser.addChoosableFileFilter(new FileFilter() {
			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) return true;

				String ext = Utils.getFileExtension(f);
				return CrebeIO.FILE_EXT.equalsIgnoreCase(ext);
			}

			@Override
			public String getDescription() {
				return CrebeIO.FILE_EXT.toUpperCase() + " (serialized creature dump)";
			}

		});
		
		exportButt.addActionListener((ActionEvent e) -> {
			/*
			EXPORT CREATURE
			*/
			try {
				CreatureListElement c = (CreatureListElement)list.getSelectedValue();
				exportFileChooser.setSelectedFile(new File(Integer.toHexString(c.creature.hashCode())));
				
				if (exportFileChooser.showSaveDialog(EvolvedListGUI.this) == JFileChooser.APPROVE_OPTION) {
					File file = exportFileChooser.getSelectedFile();
					if (!CrebeIO.FILE_EXT.equalsIgnoreCase(Utils.getFileExtension(file))) {
						file = new File(file.getAbsolutePath() + "." + CrebeIO.FILE_EXT);
					}
					
					if (file.exists()) {
						int res = JOptionPane.showConfirmDialog(EvolvedListGUI.this, "File \"" + file.getName() + "\" already exists. Overwrite file?", "Overwrite?",
								JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
						
						if (res != JOptionPane.YES_OPTION) {
							return;
						}
					}
					
					CrebeIO.exportCreature(c.creature, file);
				}

			} catch (HeadlessException | IOException ex) {
				System.err.println(ex);
				JOptionPane.showMessageDialog(EvolvedListGUI.this, ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		});
		
		JButton importButt = new JButton("Import creature");
		
		importFileChooser = new JFileChooser(new File("."));
		importFileChooser.setMultiSelectionEnabled(false);
		importFileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
		importFileChooser.setAcceptAllFileFilterUsed(false);
		importFileChooser.addChoosableFileFilter(new FileFilter() {
			@Override
			public boolean accept(File f) {
				if (f.isDirectory()) return true;

				String ext = Utils.getFileExtension(f);
				return CrebeIO.FILE_EXT.equalsIgnoreCase(ext);
			}

			@Override
			public String getDescription() {
				return CrebeIO.FILE_EXT.toUpperCase() + " (serialized creature dump)";
			}

		});
		
		importButt.addActionListener((ActionEvent e) -> {
			/*
			IMPORT CREATURE
			*/

			if (importFileChooser.showOpenDialog(EvolvedListGUI.this) == JFileChooser.APPROVE_OPTION) {
				File file = importFileChooser.getSelectedFile();
				if (!file.exists() || file.isDirectory()) return;

				try {
					Creature c = CrebeIO.importCreature(file);
					addCreatureToList(c, file.getName());
				} catch (IOException | ClassNotFoundException ex) {
					System.err.println(ex);
					JOptionPane.showMessageDialog(EvolvedListGUI.this, "Unknown data format:\n" + ex.getMessage() +
							"\nThe dump was probably created in an earlier version of the software.", "Error", JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		
		pauseButt = new JButton("Pause");
		resumeButt = new JButton("Resume");
		
		pauseButt.addActionListener((ActionEvent e) -> {
			currentEA.pause();
			pauseButt.setEnabled(false);
			resumeButt.setEnabled(true);
		});
		
		resumeButt.addActionListener((ActionEvent e) -> {
			currentEA.resume();
			pauseButt.setEnabled(true);
			resumeButt.setEnabled(false);
		});
		
		pauseButt.setEnabled(false);
		resumeButt.setEnabled(false);
		
		this.controlContentPane = new JPanel();
		controlContentPane.setLayout(new BoxLayout(controlContentPane, BoxLayout.Y_AXIS));
		
		JPanel buttPanel = new JPanel(new GridLayout(3, 2, 2, 2));
		buttPanel.setMaximumSize(new Dimension(10000, 120));
		buttPanel.add(exportButt);
		buttPanel.add(importButt);
		buttPanel.add(pauseButt);
		buttPanel.add(resumeButt);
		
		stopButt = new JButton("Stop");
		newEvoButt = new JButton("New evolution...");
		
		stopButt.setEnabled(false);
		stopButt.addActionListener(l -> {
			if (currentEA != null) {
				currentEA.resume();
				currentEA.abort();
			}
			CreativeBehaviour.aborted = true;
			stopButt.setEnabled(false);
			newEvoButt.setEnabled(true);
			pauseButt.setEnabled(false);
			resumeButt.setEnabled(false);
			resetInstruments();
		});
		buttPanel.add(stopButt);
		
		
		newEvoButt.addActionListener(l -> {
			SetupWindow.showSetupWindow(this);
		});
		buttPanel.add(newEvoButt);
		
		
		controlContentPane.add(buttPanel);
		
		JPanel instrumentPanel = new JPanel();
		instrumentPanel.setLayout(new BoxLayout(instrumentPanel, BoxLayout.Y_AXIS));
		JLabel label = new JLabel("Generational data:");
		instrumentPanel.add(label);
		
		maxFitnessInstrument = new DoubleInstrument("Max. fitness: ");
		avgFitnessInstrument = new DoubleInstrument("Avg. fitness: ");
		stdFitnessInstrument = new DoubleInstrument("Std. fitness: ");
		genInstrument = new Instrument<>("Generation: ");
		instrumentPanel.add(maxFitnessInstrument);
		instrumentPanel.add(avgFitnessInstrument);
		instrumentPanel.add(stdFitnessInstrument);
		instrumentPanel.add(genInstrument);
		
		genProgressBar = new JProgressBar();
		genProgressBar.setStringPainted(true);
		instrumentPanel.add(genProgressBar);
		
		JPanel fill = new JPanel();
		fill.setMaximumSize(new Dimension(0, 5));
		instrumentPanel.add(fill);
		
		totalRunProgressBar = new JProgressBar();
		totalRunProgressBar.setStringPainted(true);
		instrumentPanel.add(totalRunProgressBar);
		
		fill = new JPanel();
		fill.setMaximumSize(new Dimension(0, 30));
		
		instrumentPanel.add(fill);
		
		controlContentPane.add(instrumentPanel);
		
		add(controlContentPane);
		pack();
	}
	
	
	public void setCurrentEA(EA ea) {
		currentEA = ea;
		pauseButt.setEnabled(false);
		resumeButt.setEnabled(false);
		newEvoButt.setEnabled(false);
		stopButt.setEnabled(true);
		
		if (ea != null) {
			if (ea.isPaused()) resumeButt.setEnabled(true);
			else pauseButt.setEnabled(true);
			
			genProgressBar.setMaximum(ea.getGenerationLimit());
			totalRunProgressBar.setMaximum(ea.getGenerationLimit() * CreativeBehaviour.runs);
			totalRunProgressBar.setValue(currentEA.getCurrentGeneration() + CreativeBehaviour.currentRun * currentEA.getGenerationLimit());
			
			ea.addListener(new OnChangeListener() {
				@Override
				public void handle(OnChangedEvent e) {
					if (e == OnChangedEvent.NEW_GENERATION) {
						genInstrument.updateData(currentEA.getCurrentGeneration());
						genProgressBar.setValue(currentEA.getCurrentGeneration());
						totalRunProgressBar.setValue(currentEA.getCurrentGeneration() + CreativeBehaviour.currentRun * currentEA.getGenerationLimit());
					} else if ((e == OnChangedEvent.FITNESS_LIMIT_REACHED ||
							e == OnChangedEvent.GENERATION_LIMIT_REACHED) &&
							CreativeBehaviour.currentRun == CreativeBehaviour.runs-1) {
						stopButt.setEnabled(false);
						newEvoButt.setEnabled(true);
						pauseButt.setEnabled(false);
						resumeButt.setEnabled(false);
						if (currentEA.isAborted()) resetInstruments();
					}
				}
			});
		}
	}
	
	
	public void updateMaxFitness(double maxFitness) {
		maxFitnessInstrument.updateData(maxFitness);
	}
	
	
	public void updateAvgFitness(double avgFitness) {
		avgFitnessInstrument.updateData(avgFitness);
	}
	
	
	public void updateStdFitness(double stdFitness) {
		stdFitnessInstrument.updateData(stdFitness);
	}
	
	
	public void resetInstruments() {
		maxFitnessInstrument.reset();
		avgFitnessInstrument.reset();
		stdFitnessInstrument.reset();
		genInstrument.reset();
		genProgressBar.setValue(0);
		totalRunProgressBar.setValue(0);
	}
	
	
	public void addCreatureToList(Creature c) {
		addCreatureToList(c, null);
	}
	
	
	public void addCreatureToList(Creature c, String importName) {
		int i = listModel.size();
		listModel.insertElementAt(new CreatureListElement(c, importName), i);
	}

	
	public void clearCreatures() {
		listModel.removeRange(1, listModel.getSize()-1);
	}

	
	
	private static class CreatureListElement implements Comparable<CreatureListElement> {
		
		public final Creature creature;
		public final double fitness;
		public final double upkeep;
		public final String importName;
		
		
		public CreatureListElement() {
			this.creature = null;
			this.fitness = 0;
			this.upkeep = 0;
			this.importName = "";
		}
		
		public CreatureListElement(Creature creature) {
			this(creature, "");
		}

		public CreatureListElement(Creature creature, String importName) {
			this.fitness = creature.getFitness();
			this.upkeep = creature.getUpkeep();
			this.creature = creature;
			this.importName = importName == null ? "" : importName;
		}

		@Override
		public String toString() {
			return String.format(Locale.ENGLISH, "%-12h%-12.3f%-12.1f%s", creature, fitness, upkeep, importName);
		}
		
		@Override
		public int compareTo(CreatureListElement o) {
			if (fitness > o.fitness) return 1;
			if (fitness < o.fitness) return -1;
			return 0;
		}
	}
	
}


class Instrument<T> extends JPanel {
	
	private T data;
	private final JLabel label;
	private final JLabel dataLabel;
	
	public Instrument(String label) {
		this.label = new JLabel(label);
		this.label.setFont(FONT);
		this.dataLabel = new JLabel("N/A");
		this.dataLabel.setFont(FONT);
		
		setLayout(new BorderLayout());
		add(this.label, BorderLayout.WEST);
		add(this.dataLabel, BorderLayout.EAST);
		
		setMaximumSize(new Dimension(1000, FONT.getSize() * 2));
	}
	
	public void updateData(T data) {
		this.data = data;
		String dataStr = dataToString(data);
		dataLabel.setText(dataStr);
	}
	
	public void reset() {
		this.dataLabel.setText("N/A");
	}
	
	protected String dataToString(T data) {
		return data.toString();
	}
	
}


class DoubleInstrument extends Instrument<Double> {
	public DoubleInstrument(String label) {
		super(label);
	}

	@Override
	protected String dataToString(Double data) {
		return String.format(Locale.ENGLISH, "%.2f", data);
	}
}