/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package creativebehaviour.graphics.evolvedlistgui;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.ea.BrainEA;
import creativebehaviour.simulation.WorldType;
import static creativebehaviour.simulation.WorldType.ARENA_WORLD;
import static creativebehaviour.simulation.WorldType.FOOD_WORLD;
import static creativebehaviour.simulation.WorldType.FOOD_WORLD_EDGE_SPAWN;
import static creativebehaviour.simulation.WorldType.POISON_WORLD;
import creativebehaviour.util.Utils;
import eval.util.Config;
import java.awt.Component;
import java.awt.Frame;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.filechooser.FileFilter;

/**
 *
 * @author bjorn
 */
public class SetupWindow extends javax.swing.JDialog {
	
	private static SetupWindow instance;
	
	private final JFileChooser fileChooser;
	
	public static void showSetupWindow(Frame parent) {
		if (instance == null) instance = new SetupWindow(parent, true);
		instance.setLocationRelativeTo(parent);
		instance.setVisible(true);
	}
	
	/**
	 * Creates new form SetupWindow
	 */
	private SetupWindow(java.awt.Frame parent, boolean modal) {
		super(parent, modal);
		initComponents();
		
		fileChooser = new JFileChooser(new File("."));
		fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
		fileChooser.setAcceptAllFileFilterUsed(false);
		fileChooser.addChoosableFileFilter(new FileFilter() {
			@Override
			public boolean accept(File f) {
				return f.isDirectory() || "txt".equalsIgnoreCase(Utils.getFileExtension(f));
			}

			@Override
			public String getDescription() {
				return "txt (entity file)";
			}

		});
		
		fullGenSpinner.setModel(new SpinnerNumberModel(20, 1, 999999999, 1));
		fullPopSpinner.setModel(new SpinnerNumberModel(20, 1, 999999999, 1));
		behaviourGenSpinner.setModel(new SpinnerNumberModel(20, 1, 999999999, 1));
		behaviourPopSpinner.setModel(new SpinnerNumberModel(20, 1, 999999999, 1));
		runsSpinner.setModel(new SpinnerNumberModel(1, 1, 999999999, 1));
		
		Config crebecfg = CreativeBehaviour.crebeCfg;
		runsSpinner.setValue(1);
		scenarioCombobox.setSelectedItem(WorldType.valueOf(crebecfg.get("world_type")));
		
		Config creaturecfg = CreativeBehaviour.creatureCfg;
		fullPopSpinner.setValue(creaturecfg.getInt("population_size"));
		fullGenSpinner.setValue(creaturecfg.getInt("max_generations"));
		
		updateBehaviourSpinners();
	}
	
	private void updateBehaviourSpinners() {
		BrainEA.Method brainModule = (BrainEA.Method) brainModuleCombobox.getSelectedItem();
		
		Config behaviourcfg = null;
		switch (brainModule) {
			case CTRNN:
				behaviourcfg = CreativeBehaviour.weightsCfg;;
				break;
			case FUZZY_LOGIC:
				behaviourcfg = CreativeBehaviour.fuzzyCfg;;
				break;
			case NEAT:
				behaviourcfg = CreativeBehaviour.neatCfg;;
				break;
			default:
				throw new AssertionError(brainModule.name());
		}
		
		behaviourPopSpinner.setValue(behaviourcfg.getInt("population_size"));
		behaviourGenSpinner.setValue(behaviourcfg.getInt("max_generations"));
	}
	
	private ComboBoxModel<WorldType> getScenarios() {
		WorldType[] types = {FOOD_WORLD, FOOD_WORLD_EDGE_SPAWN, POISON_WORLD, ARENA_WORLD};
		return new DefaultComboBoxModel<>(types);
	}

	/**
	 * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The content of this method is always regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        runModeGroup = new javax.swing.ButtonGroup();
        jLabel1 = new javax.swing.JLabel();
        brainModuleCombobox = new javax.swing.JComboBox<>();
        jLabel2 = new javax.swing.JLabel();
        scenarioCombobox = new javax.swing.JComboBox<>();
        jLabel3 = new javax.swing.JLabel();
        fullRadioButton = new javax.swing.JRadioButton();
        designedRadioButton = new javax.swing.JRadioButton();
        designedCreatureField = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        fullGenSpinner = new javax.swing.JSpinner();
        behaviourGenSpinner = new javax.swing.JSpinner();
        jLabel5 = new javax.swing.JLabel();
        fullPopSpinner = new javax.swing.JSpinner();
        behaviourPopSpinner = new javax.swing.JSpinner();
        browseLabel = new javax.swing.JLabel();
        browseEntFileButton = new javax.swing.JButton();
        jLabel6 = new javax.swing.JLabel();
        runsSpinner = new javax.swing.JSpinner();
        dumpFitnessCheckbox = new javax.swing.JCheckBox();
        startButton = new javax.swing.JButton();
        multithreadCheckbox = new javax.swing.JCheckBox();
        cancelButton = new javax.swing.JButton();
        jLabel7 = new javax.swing.JLabel();
        physicalEaLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("New evolution setup");
        setResizable(false);

        jLabel1.setText("Brain module:");

        brainModuleCombobox.setModel(new DefaultComboBoxModel(BrainEA.Method.values()));
        brainModuleCombobox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                brainModuleComboboxActionPerformed(evt);
            }
        });

        jLabel2.setText("World scenario:");

        scenarioCombobox.setModel(getScenarios());

        jLabel3.setText("Run mode:");

        runModeGroup.add(fullRadioButton);
        fullRadioButton.setSelected(true);
        fullRadioButton.setText("Full");
        fullRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fullRadioButtonActionPerformed(evt);
            }
        });

        runModeGroup.add(designedRadioButton);
        designedRadioButton.setText("Designed");
        designedRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                designedRadioButtonActionPerformed(evt);
            }
        });

        designedCreatureField.setEnabled(false);
        designedCreatureField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                designedCreatureFieldActionPerformed(evt);
            }
        });

        jLabel4.setText("Generations:");

        fullGenSpinner.setValue(20);

        behaviourGenSpinner.setValue(20);

        jLabel5.setText("Population size:");

        fullPopSpinner.setValue(20);

        behaviourPopSpinner.setValue(20);

        browseLabel.setText("Entity file:");
        browseLabel.setEnabled(false);

        browseEntFileButton.setText("Browse...");
        browseEntFileButton.setEnabled(false);
        browseEntFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseEntFileButtonActionPerformed(evt);
            }
        });

        jLabel6.setText("Number of runs:");

        runsSpinner.setValue(1);

        dumpFitnessCheckbox.setSelected(true);
        dumpFitnessCheckbox.setText("Dump fitness data to disk");

        startButton.setText("Start");
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });

        multithreadCheckbox.setSelected(true);
        multithreadCheckbox.setText("Use multiple threads");

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        jLabel7.setText("Behaviour EA:");

        physicalEaLabel.setText("Physical evolution EA:");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(startButton, javax.swing.GroupLayout.PREFERRED_SIZE, 118, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(cancelButton))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel2)
                                    .addComponent(jLabel3)
                                    .addComponent(jLabel1))
                                .addGap(70, 70, 70)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(jLabel7)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addComponent(physicalEaLabel)
                                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                                .addComponent(fullGenSpinner, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addComponent(fullPopSpinner, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE))))
                                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                        .addComponent(designedRadioButton)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                        .addComponent(fullRadioButton)
                                        .addGap(49, 49, 49))
                                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(scenarioCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, 320, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(brainModuleCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, 320, javax.swing.GroupLayout.PREFERRED_SIZE))))
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel6)
                                    .addComponent(dumpFitnessCheckbox)
                                    .addComponent(multithreadCheckbox)
                                    .addGroup(layout.createSequentialGroup()
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addComponent(jLabel4)
                                            .addComponent(jLabel5)
                                            .addComponent(browseLabel))
                                        .addGap(70, 70, 70)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addComponent(designedCreatureField, javax.swing.GroupLayout.PREFERRED_SIZE, 135, javax.swing.GroupLayout.PREFERRED_SIZE)
                                            .addComponent(runsSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)
                                            .addComponent(browseEntFileButton)
                                            .addComponent(behaviourGenSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE)
                                            .addComponent(behaviourPopSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE))))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)))
                        .addGap(0, 1, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(brainModuleCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(scenarioCombobox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel3)
                        .addComponent(designedRadioButton))
                    .addComponent(fullRadioButton))
                .addGap(26, 26, 26)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(physicalEaLabel))
                .addGap(18, 18, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addComponent(jLabel5)
                        .addGroup(layout.createSequentialGroup()
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(behaviourGenSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel4))
                            .addGap(18, 18, 18)
                            .addComponent(behaviourPopSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(fullGenSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(fullPopSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(21, 21, 21)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(browseLabel)
                    .addComponent(designedCreatureField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(browseEntFileButton)
                .addGap(35, 35, 35)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(runsSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addComponent(dumpFitnessCheckbox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(multithreadCheckbox)
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(startButton)
                    .addComponent(cancelButton))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void brainModuleComboboxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_brainModuleComboboxActionPerformed
		updateBehaviourSpinners();
    }//GEN-LAST:event_brainModuleComboboxActionPerformed

    private void fullRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fullRadioButtonActionPerformed
		fullGenSpinner.setEnabled(true);
		fullPopSpinner.setEnabled(true);
		designedCreatureField.setEnabled(false);
		browseEntFileButton.setEnabled(false);
		browseLabel.setEnabled(false);
		physicalEaLabel.setEnabled(true);
    }//GEN-LAST:event_fullRadioButtonActionPerformed

    private void designedCreatureFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_designedCreatureFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_designedCreatureFieldActionPerformed

    private void browseEntFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseEntFileButtonActionPerformed
		if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			designedCreatureField.setText(file.getAbsolutePath());
		}
    }//GEN-LAST:event_browseEntFileButtonActionPerformed

    private void designedRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_designedRadioButtonActionPerformed
        fullGenSpinner.setEnabled(false);
		fullPopSpinner.setEnabled(false);
		designedCreatureField.setEnabled(true);
		browseEntFileButton.setEnabled(true);
		browseLabel.setEnabled(true);
		physicalEaLabel.setEnabled(false);
    }//GEN-LAST:event_designedRadioButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        setVisible(false);
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startButtonActionPerformed
		if (designedRadioButton.isSelected()) {
			File designedFile = new File(designedCreatureField.getText());
			if (!designedFile.exists()) {
				JOptionPane.showMessageDialog(this, "Please specify a file in the \"Entity file:\" field when selecting a designed run.", "Missing value", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
		}
		
		Config crebecfg = CreativeBehaviour.crebeCfg;
		Config creaturecfg = CreativeBehaviour.creatureCfg;
		
		BrainEA.Method brainModule = (BrainEA.Method) brainModuleCombobox.getSelectedItem();
		CreativeBehaviour.brainMethod = brainModule;
		crebecfg.set("world_type", scenarioCombobox.getSelectedItem().toString());
		CreativeBehaviour.worldType = WorldType.valueOf(scenarioCombobox.getSelectedItem().toString());
		CreativeBehaviour.runMode = designedRadioButton.isSelected() ? CreativeBehaviour.RunMode.DESIGNED : CreativeBehaviour.RunMode.FULL;
		
		Config behaviourcfg = null;
		switch (brainModule) {
			case CTRNN:
				behaviourcfg = CreativeBehaviour.weightsCfg;;
				break;
			case FUZZY_LOGIC:
				behaviourcfg = CreativeBehaviour.fuzzyCfg;;
				break;
			case NEAT:
				behaviourcfg = CreativeBehaviour.neatCfg;;
				break;
			default:
				throw new AssertionError(brainModule.name());
		}
		
		behaviourcfg.set("max_generations", String.valueOf(behaviourGenSpinner.getValue()));
		behaviourcfg.set("population_size", String.valueOf(behaviourPopSpinner.getValue()));
		
		if (fullRadioButton.isSelected()) {
			creaturecfg.set("max_generations", String.valueOf(fullGenSpinner.getValue()));
			creaturecfg.set("population_size", String.valueOf(fullPopSpinner.getValue()));
		}
		
		if (designedRadioButton.isSelected()) {
			CreativeBehaviour.designedCreatureFileName = designedCreatureField.getText();
		}
		CreativeBehaviour.runs = Integer.parseInt(String.valueOf(runsSpinner.getValue()));
		CreativeBehaviour.dumpFitness = dumpFitnessCheckbox.isSelected();
		
		if (dumpFitnessCheckbox.isSelected()) {
			SimpleDateFormat dateFormat = new SimpleDateFormat("ddMMyy_HHmmss");
			CreativeBehaviour.runName = dateFormat.format(new Date());
		}
		
		String threads = multithreadCheckbox.isSelected() ? "0" : "1";
		creaturecfg.set("threads", threads);
		behaviourcfg.set("threads", threads);
		
		try {
			crebecfg.store();
			creaturecfg.store();
			behaviourcfg.store();
			setVisible(false);
			CreativeBehaviour.runAsync();
		} catch (Exception e) {
			System.err.println(e);
			JOptionPane.showMessageDialog(this, "An error occurred while writing configuration parameters to disk.\n" + e.toString(), "Error", JOptionPane.ERROR_MESSAGE);
		}
    }//GEN-LAST:event_startButtonActionPerformed

	/**
	 * @param args the command line arguments
	 */
//	public static void main(String args[]) {
//		/* Set the Nimbus look and feel */
//		//<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
//		/* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
//         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
//		 */
//		try {
//			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
//				if ("Nimbus".equals(info.getName())) {
//					javax.swing.UIManager.setLookAndFeel(info.getClassName());
//					break;
//				}
//			}
//		} catch (ClassNotFoundException ex) {
//			java.util.logging.Logger.getLogger(SetupWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//		} catch (InstantiationException ex) {
//			java.util.logging.Logger.getLogger(SetupWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//		} catch (IllegalAccessException ex) {
//			java.util.logging.Logger.getLogger(SetupWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
//			java.util.logging.Logger.getLogger(SetupWindow.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//		}
//		//</editor-fold>
//
//		/* Create and display the dialog */
//		java.awt.EventQueue.invokeLater(new Runnable() {
//			public void run() {
//				SetupWindow dialog = new SetupWindow(new javax.swing.JFrame(), true);
//				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
//					@Override
//					public void windowClosing(java.awt.event.WindowEvent e) {
//						System.exit(0);
//					}
//				});
//				dialog.setVisible(true);
//			}
//		});
//	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JSpinner behaviourGenSpinner;
    private javax.swing.JSpinner behaviourPopSpinner;
    private javax.swing.JComboBox<String> brainModuleCombobox;
    private javax.swing.JButton browseEntFileButton;
    private javax.swing.JLabel browseLabel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JTextField designedCreatureField;
    private javax.swing.JRadioButton designedRadioButton;
    private javax.swing.JCheckBox dumpFitnessCheckbox;
    private javax.swing.JSpinner fullGenSpinner;
    private javax.swing.JSpinner fullPopSpinner;
    private javax.swing.JRadioButton fullRadioButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JCheckBox multithreadCheckbox;
    private javax.swing.JLabel physicalEaLabel;
    private javax.swing.ButtonGroup runModeGroup;
    private javax.swing.JSpinner runsSpinner;
    private javax.swing.JComboBox<WorldType> scenarioCombobox;
    private javax.swing.JButton startButton;
    // End of variables declaration//GEN-END:variables
}
