package creativebehaviour.models.ann;

import anune.ANN;
//import creativebehaviour.graphics.evolvedlistgui.GenotypeWrapper;
import creativebehaviour.models.Creature;
import creativebehaviour.models.physical.TreeGenotype;
import eval.models.Genotype;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class ANNCreature extends Creature {
	
	/** The genotype for this type of creature. */
	private final WeightsGenotype weightsGenotype;
	
	/** The Neural network. */
	private final ANN brain;
	
	/** Hidden layer configuration of ANN. */
	private final int[] annHiddenConfiguration;
	
	
	public ANNCreature(TreeGenotype treeGenotype, WeightsGenotype weightsGenotype) {
		this(treeGenotype, weightsGenotype, NetworkFactory.ANN_HIDDEN_CONFIGURATION);
	}
	
	
	public ANNCreature(TreeGenotype treeGenotype, WeightsGenotype weightsGenotype, int[] annHidddenConfig) {
		super(treeGenotype, weightsGenotype);
		
		this.weightsGenotype = weightsGenotype;
		this.annHiddenConfiguration = new int[annHidddenConfig.length];
		
		System.arraycopy(annHidddenConfig, 0, this.annHiddenConfiguration, 0, annHidddenConfig.length);
		
		if (weightsGenotype.getValues() == null)
			this.brain = NetworkFactory.makeANN(treeGenotype.tree, annHidddenConfig);
		else
			this.brain = NetworkFactory.makeANN(treeGenotype.tree, annHidddenConfig, weightsGenotype.getValues());
	}
	
	
//	public ANNCreature(GenotypeWrapper wrapper) {
//		this(wrapper.tg, wrapper.wg, wrapper.annHiddenConfig);
//	}
	
	
	/**
	 * Copy-constructor.
	 * @param original 
	 */
	public ANNCreature(ANNCreature original) {
		this(new TreeGenotype(original.treeGenotype), new WeightsGenotype(original.weightsGenotype), original.annHiddenConfiguration);
		setEqual(original);
	}
	
	
//	public GenotypeWrapper createGenotypeWrapper() {
//		return new GenotypeWrapper(weightsGenotype, treeGenotype, fitness, annHiddenConfiguration);
//	}
	
	
	@Override
	protected void customCreatureTick(float[] sensorData, float[] actuatorBuffer) {
		float[] annOutput = brain.run(sensorData);
		assert annOutput.length == actuatorBuffer.length : annOutput.length + " != " + actuatorBuffer.length;
		System.arraycopy(annOutput, 0, actuatorBuffer, 0, actuatorBuffer.length);
	}

	
	@Override
	public Creature copy() {
		return new ANNCreature(this);
	}

	
	@Override
	public WeightsGenotype getGenotype() {
		return weightsGenotype;
	}
	
}
