/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package creativebehaviour.models.ann;

import anune.ANN;
import anune.Neuron;
import creativebehaviour.CreativeBehaviour;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.models.BehaviourGenotype;
import creativebehaviour.models.physical.TreeGenotype;
import creativebehaviour.util.Utils;
import creativebehaviour.util.tree.Node;
import eval.models.Crossover;
import eval.models.Genotype;
import java.util.ArrayList;
import java.util.Stack;

/**
 *
 * @author Endre
 */
public class WeightsGenotype extends BehaviourGenotype {
	
	private static final double weight_mutation_std = CreativeBehaviour.weightsCfg.getDouble("weight_mutation_std");
	
	public static final int WEIGHTS = 0;
	public static final int BIASES = 1;
	public static final int TAUS = 2;
	public static final int GAINS = 3;
	
	private float[][] weights = new float[4][];
	
	
	
	public WeightsGenotype(int nrOfWeights, int nrOfBiases, int nrOfTaus, int nrOfGains) {
		weights[WEIGHTS] = new float[nrOfWeights];
		weights[BIASES] = new float[nrOfBiases];
		weights[TAUS] = new float[nrOfTaus];
		weights[GAINS] = new float[nrOfGains];
		
		randomizeWeights();
	}
	
	
	public WeightsGenotype(float[][] weights) {
		this.weights = weights;
	}
	
	
	public WeightsGenotype(TreeGenotype tg) {
		int[] annConfig = new int[NetworkFactory.ANN_HIDDEN_CONFIGURATION.length + 2];
		for (Wrapper entwrapper : tg.tree) {
			if (entwrapper.isSensor()) annConfig[0] += entwrapper.entType.getInputLenght();
			if (entwrapper.isActuator()) annConfig[annConfig.length-1] += entwrapper.entType.getOutputLenght();
		}
		for (int i = 0; i < NetworkFactory.ANN_HIDDEN_CONFIGURATION.length; i++) {
			annConfig[i+1] = NetworkFactory.ANN_HIDDEN_CONFIGURATION[i];
		}
		int numberOfNodes = 0;
		for (int i = 0; i < annConfig.length; i++) {
			numberOfNodes += annConfig[i];
		}
		int numberOfWeights = 0;
		for (int i = 1; i < annConfig.length; i++) {
			numberOfWeights += annConfig[i-1]*annConfig[i] + annConfig[i]*annConfig[i];
		}
		weights[WEIGHTS] = new float[numberOfWeights];
		weights[BIASES] = new float[numberOfNodes];
		weights[TAUS] = new float[numberOfNodes];
		weights[GAINS] = new float[numberOfNodes];
		
		randomizeWeights();
	}
	
	
	/**
	 * Copy constructor.
	 * @param orig 
	 */
	public WeightsGenotype(WeightsGenotype orig) {
		for (int i = 0; i < weights.length; i++) {
			int len = orig.weights[i].length;
			weights[i] = new float[len];
			System.arraycopy(orig.weights[i], 0, weights[i], 0, len);
		}
	}
	
	
	private void randomizeWeights() {
		for (int i = 0; i < weights[WEIGHTS].length; i++) {
			weights[WEIGHTS][i] = (Utils.randomFloat() - 0.5f) * 10; // -5, 5
		}
		for (int i = 0; i < weights[BIASES].length; i++) {
			weights[BIASES][i] = Utils.randomFloat() * 6 - 3; // -3, 3
		}
		for (int i = 0; i < weights[TAUS].length; i++) {
			weights[TAUS][i] = Utils.randomFloat() + 1.0f; // 1, 2
		}
		for (int i = 0; i < weights[GAINS].length; i++) {
			weights[GAINS][i] = Utils.randomFloat() * 4 + 1; // 1, 5
		}
	}
	
	
	@Override
	public Genotype copy() {
		return new WeightsGenotype(this);
	}
	

	@Override
	public Genotype crossover(Genotype g, Crossover type) {
		switch (type) {
			case ONE_POINT:
				return onePointCrossover((WeightsGenotype) g);
			case UNIFORM:
				return uniformCrossover((WeightsGenotype) g);
			default:
				throw new AssertionError(type.name());
			
		}
	}
	

	@Override
	public void mutate(double prob) {
		for (int i = 0; i < this.weights.length; i++) {
			for (int j = 0; j < this.weights[i].length; j++) {
				if (Utils.randomDouble() < prob) {
					this.weights[i][j] += Utils.randomGausian(0.0, weight_mutation_std);
				}
			}
		}
	}
	
	
	public float getWeight(int type, int i) {
		return this.weights[type][i];
	}
	
	public float[] getWeights() {
		return this.weights[WEIGHTS];
	}
	
	public float[] getBiases() {
		return this.weights[BIASES];
	}
	
	public float[] getTaus() {
		return this.weights[TAUS];
	}
	
	public float[] getGains() {
		return this.weights[GAINS];
	}
	
	public float[][] getValues() {
		return this.weights;
	}
	
	
	private WeightsGenotype onePointCrossover(WeightsGenotype g) {
		int splitIndex = Utils.randomInt(this.weights[WEIGHTS].length);
		float[][] weights = new float[4][];
		for (int i = 0; i < this.weights.length; i++) {
			weights[i] = new float[this.weights[i].length];
		}
		System.arraycopy(this.weights[WEIGHTS], 0, weights[WEIGHTS], 0, splitIndex);
		System.arraycopy(g.weights[WEIGHTS], splitIndex, weights[WEIGHTS], splitIndex, g.weights[WEIGHTS].length-splitIndex);
		
		splitIndex = Utils.randomInt(this.weights[BIASES].length);
		System.arraycopy(this.weights[BIASES], 0, weights[BIASES], 0, splitIndex);
		System.arraycopy(g.weights[BIASES], splitIndex, weights[BIASES], splitIndex, g.weights[BIASES].length-splitIndex);
		
		splitIndex = Utils.randomInt(this.weights[TAUS].length);
		System.arraycopy(this.weights[TAUS], 0, weights[TAUS], 0, splitIndex);
		System.arraycopy(g.weights[TAUS], splitIndex, weights[TAUS], splitIndex, g.weights[TAUS].length-splitIndex);
		
		splitIndex = Utils.randomInt(this.weights[GAINS].length);
		System.arraycopy(this.weights[GAINS], 0, weights[GAINS], 0, splitIndex);
		System.arraycopy(g.weights[GAINS], splitIndex, weights[GAINS], splitIndex, g.weights[GAINS].length-splitIndex);
		
		return new WeightsGenotype(weights);
	}
	
	
	private WeightsGenotype uniformCrossover(WeightsGenotype g) {
		float[][] weights = new float[4][];
		weights[WEIGHTS] = new float[this.weights[WEIGHTS].length];
		weights[BIASES] = new float[this.weights[BIASES].length];
		weights[TAUS] = new float[this.weights[TAUS].length];
		weights[GAINS] = new float[this.weights[GAINS].length];
		
		for (int i = 0; i < weights.length; i++) {
			for (int j = 0; j < weights[i].length; j++) {
				if (Utils.randomEvent(0.5)) weights[i][j] = this.weights[i][j];
				else weights[i][j] = g.getWeight(i, j);
			}
		}
		
		return new WeightsGenotype(weights);
	}

	@Override
	public BehaviourGenotype treeGenotypeOnePointCrossover(TreeGenotype child, TreeGenotype parent1, TreeGenotype parent2, int splitIndex) {
		ANN parent1Network = NetworkFactory.makeANN(parent1.tree, ((WeightsGenotype)parent1.behaviourGenotype).weights);
		ANN parent2Network = NetworkFactory.makeANN(parent2.tree, ((WeightsGenotype)parent2.behaviourGenotype).weights);
		
		ANN childNetwork = NetworkFactory.makeANN(child.tree);
		
		int size = child.rootEnt().ENT_ATTACH_POINTS;
		
		float ratio = splitIndex / (float) size;
		ArrayList<Node<Wrapper>> inputNodes = new ArrayList<>();
		ArrayList<Node<Wrapper>> outputNodes = new ArrayList<>();
		Stack<Node<Wrapper>> stack = new Stack<>();
		stack.push(child.rootNode());
		
		// Body input
		for (int i = 0; i < childNetwork.getLayers().get(1).neurons.size(); i++) {
				childNetwork.getLayers().get(1).neurons.get(i).weights[0] 
					= (parent1Network.getLayers().get(1).neurons.get(i).weights[0]*ratio + parent2Network.getLayers().get(1).neurons.get(i).weights[0]*(1-ratio));
		}
		int outputLayerIndex = parent1Network.getConfiguration()[parent1Network.getConfiguration().length-1];
		// Add all inputs from parent1
		while (!stack.isEmpty()) {
			Node<Wrapper> node = stack.pop();
			
			if (node.obj.isSensor()) inputNodes.add(node);
			if (node.obj.isActuator()) outputNodes.add(node);
			
			for (int i = 0; i < splitIndex; i++) {
				stack.add(node.children.get(i));
			}
		}
		
		for (int i = 1; i < inputNodes.size(); i++) {
			for (int j = 0; j < childNetwork.getLayers().get(1).neurons.size(); j++) {
				childNetwork.getLayers().get(1).neurons.get(j).weights[i] = parent1Network.getLayers().get(1).neurons.get(j).weights[i];
			}
		}
		// output parent1
		for (int i = 0; i < outputNodes.size(); i++) {
			for (int j = 0; j < parent1Network.getLayers().get(outputLayerIndex).neurons.get(i).weights.length; j++) {
				childNetwork.getLayers().get(1).neurons.get(outputLayerIndex).weights[j] = parent1Network.getLayers().get(outputLayerIndex).neurons.get(i).weights[j];
			}
		}
		int prevInputNodesSize = inputNodes.size();
		inputNodes.clear();
		int prevOutputNodesSize = outputNodes.size();
		outputNodes.clear();
		// Add all inputs from parent2
		stack.push(child.rootNode());
		while (!stack.isEmpty()) {
			Node<Wrapper> node = stack.pop();
			
			if (node.obj.isSensor()) inputNodes.add(node);
			if (node.obj.isActuator()) outputNodes.add(node);
			
			stack.addAll(node.children);
			for (int i = splitIndex; i < size; i++) {
				stack.add(node.children.get(i));
			}
		}
		for (int i = 0; i < inputNodes.size(); i++) {
			for (int j = 0; j < childNetwork.getLayers().get(1).neurons.size(); j++) {
				childNetwork.getLayers().get(1).neurons.get(j).weights[i+prevInputNodesSize] = parent2Network.getLayers().get(1).neurons.get(j).weights[+prevInputNodesSize];
			}
		}
		// TODO output parent2
		for (int i = 0; i < outputNodes.size(); i++) {
			for (int j = 0; j < parent2Network.getLayers().get(outputLayerIndex).neurons.get(i).weights.length; j++) {
				childNetwork.getLayers().get(1).neurons.get(outputLayerIndex).weights[j] = parent2Network.getLayers().get(outputLayerIndex).neurons.get(i).weights[j];
			}
		}
		// first hidden layer samelayerweights
		for (int i = 0; i < parent2Network.getLayers().get(1).neurons.size(); i++) {
			Neuron childNeuron = childNetwork.getLayers().get(1).neurons.get(i);
			Neuron thisNeuron = parent1Network.getLayers().get(1).neurons.get(i);
			Neuron otherNeuron = parent2Network.getLayers().get(1).neurons.get(i);
			for (int j = 0; j < otherNeuron.sameLayerWeights.length; j++) {
				childNeuron.sameLayerWeights[j] = (thisNeuron.sameLayerWeights[j]*ratio + otherNeuron.sameLayerWeights[j]*(1-ratio));
			}
			childNeuron.tau = (thisNeuron.tau*ratio + otherNeuron.tau*(1-ratio)) / 2;
			childNeuron.gain = (thisNeuron.gain*ratio + otherNeuron.gain*(1-ratio)) / 2;
			childNeuron.bias = (thisNeuron.bias*ratio + otherNeuron.bias*(1-ratio)) / 2;
			
		}
		// hidden layer weights
		for (int i = 2; i < parent1Network.getConfiguration().length-1; i++) {
			for (int j = 0; j < parent1Network.getLayers().get(i).neurons.size(); j++) {
				Neuron childNeuron = childNetwork.getLayers().get(i).neurons.get(j);
				Neuron thisNeuron = parent1Network.getLayers().get(i).neurons.get(j);
				Neuron otherNeuron = parent2Network.getLayers().get(i).neurons.get(j);
				//weights
				for (int l = 0; l < otherNeuron.weights.length; l++) {
					childNeuron.weights[j] = (thisNeuron.weights[j]*ratio + otherNeuron.weights[j]*(1-ratio));
				}
				//samelayerweights
				for (int l = 0; l < otherNeuron.sameLayerWeights.length; l++) {
					childNeuron.sameLayerWeights[j] = (thisNeuron.sameLayerWeights[j]*ratio + otherNeuron.sameLayerWeights[j]*(1-ratio));
				}
				childNeuron.tau = (thisNeuron.tau*ratio + otherNeuron.tau*(1-ratio));
				childNeuron.gain = (thisNeuron.gain*ratio + otherNeuron.gain*(1-ratio));
				childNeuron.bias = (thisNeuron.bias*ratio + otherNeuron.bias*(1-ratio));
			}
		}
		
		// Convert from ANN to WeightsGenotype
		
		float[][] weights = new float[4][];
		weights[WEIGHTS] = new float[childNetwork.getWeightsCount()];
		weights[BIASES] = new float[childNetwork.getBiasesCount()];
		weights[TAUS] = new float[childNetwork.getTausCount()];
		weights[GAINS] = new float[childNetwork.getGainsCount()];
		
		int weightsCounter = 0;
		int biasesCounter = 0;
		int tausCounter = 0;
		int gainsCounter = 0;
		for (int i = 1; i < childNetwork.getLayers().size(); i++) {
			for (Neuron n : childNetwork.getLayers().get(i).neurons) {
				// Updates outgoing weights
				for (int j = 0; j < n.weights.length; j++){
					weights[WEIGHTS][weightsCounter++] = n.weights[j];
				}
				// Updates same layer weigths
				for (int j = 0; j < n.sameLayerWeights.length; j++){
					weights[WEIGHTS][weightsCounter++] = n.sameLayerWeights[j];
				}
				weights[BIASES][biasesCounter++] = n.bias;
				weights[TAUS][tausCounter++] = n.tau;
				weights[GAINS][gainsCounter++] = n.gain;
			}
		}
		
		return new WeightsGenotype(weights); 
	}

	@Override
	public BehaviourGenotype treeGenotypeUniformCrossover(TreeGenotype child, TreeGenotype parent1, TreeGenotype parent2) {
		return null;
	}

	
}
