package creativebehaviour.models.neat;

import anune.util.Activation;
import creativebehaviour.CreativeBehaviour;
import creativebehaviour.models.neat.NeatGenotype.Edge;
import creativebehaviour.models.neat.NeatGenotype.Node;
import creativebehaviour.util.Utils;
import eval.models.Genotype;
import eval.models.Phenotype;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;

/**
 *
 * @author Endre
 */
public class NeatANN implements Serializable{
	
	public ArrayList<InputNeuron> inputNeurons;
	public ArrayList<Neuron> outputNeurons;
	
	public HashMap<Integer, Neuron> neurons;
	
	private class Neuron implements Serializable{
		
		public int id;
		public float prevVal = 0;
		protected ArrayList<Connection> connections;
		
		public Neuron(int id) {
			this.id = id;
			connections = new ArrayList<>();
		}
		
		public Neuron(Neuron orig) {
			this.id = orig.id;
			
			for (Connection conn : orig.connections) {
				this.connections.add(new Connection(conn));
			}
		}
		
		public float activate() {
			float val = 0;
			for (Connection conn : connections) {
				if (conn.isRecurrent) val += conn.fromNeuron.prevVal * conn.weight;
				else val += conn.fromNeuron.activate() * conn.weight;
			}
			switch (CreativeBehaviour.crebeCfg.get("activation_function")) {
				case "tanh": val = Activation.tanh(val); break; 
				case "atan": val = Activation.atan(val); break;
				case "softsign": val = Activation.softsign(val); break;
				case "binstep": val = Activation.binstep(val); break;
				case "binstep_inclusive_zero": val = Activation.binstep_inclusive_zero(val); break;
				case "rlu": val = Activation.rlu(val); break;
				case "mod_sigmoid": val = Activation.mod_sigmoid(val); break;
			}
			prevVal = val;
			return val;
		}
		
		public void addConnection(Connection c) {
			this.connections.add(c);
		}
	}
	
	private class InputNeuron extends Neuron {
		
		float val;
		
		public InputNeuron(int id) {
			super(id);
			val = 0;
		}
		public InputNeuron(InputNeuron orig) {
			super(orig.id);
			this.val = orig.val;
		}
		
		@Override
		public float activate() {
			return val;
		}
	}
	
	private class Connection implements Serializable{
		
		Neuron fromNeuron;
		public boolean isRecurrent = false;
		float weight;
		
		public Connection(Neuron fromNeuron, float weight) {
			this.fromNeuron = fromNeuron;
			this.weight = weight;
		}
		public Connection(Neuron fromNode) {
			this(fromNode, Utils.randomFloat());
		}
		public Connection(Connection orig) {
			this.fromNeuron = new Neuron(orig.fromNeuron);
			this. weight = orig.weight;
		}
	}
	
	public NeatANN() {
		this.inputNeurons = new ArrayList<>();
		this.outputNeurons = new ArrayList<>();
		this.neurons = new HashMap<>();
	}
	
	public NeatANN(NeatGenotype g) {
		this();
		
		for (Node n : g.nodes) {
			Neuron neuron;
			switch (n.type) {
				case INPUT: 
					neuron = new InputNeuron(n.id); 
					this.inputNeurons.add((InputNeuron) neuron);
					break;
				case OUTPUT: 
					neuron = new Neuron(n.id); 
					this.outputNeurons.add(neuron);
					break;
				case HIDDEN: 
					neuron = new Neuron(n.id); 
					break;
					
				default: neuron = new Neuron(n.id); break;
			}
			
			this.neurons.put(neuron.id, neuron);
		}
		
		for (Edge e : g.edges) {
			if (!e.disabled) {
				Neuron toNeuron = this.neurons.get(e.toNeuron);
				Neuron fromNeuron = this.neurons.get(e.fromNeuron);
				Connection c = new Connection(fromNeuron);
				c.isRecurrent = e.isRecurrent;
				toNeuron.connections.add(c);
			}
		}
		
	}
	
	public NeatANN(NeatANN orig) {
		this();
		
		for (InputNeuron n : orig.inputNeurons) {
			this.inputNeurons.add(new InputNeuron(n));
		}
		for (Neuron n : orig.outputNeurons) {
			this.outputNeurons.add(new Neuron(n));
		}
	}
	
	public float[] run(float[] sensorData) {
		assert (sensorData.length == this.inputNeurons.size());
		float[] output = new float[this.outputNeurons.size()];
		for (int i = 0; i < this.inputNeurons.size(); i++) {
			this.inputNeurons.get(i).val = sensorData[i];
		}
		for (int i = 0; i < this.outputNeurons.size(); i++) {
			output[i] = this.outputNeurons.get(i).activate();
		}
		
		return output;
	}
}
