/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package creativebehaviour.models.neat;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.models.BehaviourGenotype;
import creativebehaviour.models.physical.TreeGenotype;
import creativebehaviour.util.Utils;
import eval.models.Crossover;
import eval.models.Genotype;
import java.io.Serializable;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Deque;

/**
 *
 * @author Endre
 */
public class NeatGenotype extends BehaviourGenotype {
	
	public static final Object LOCK = new Object();
	
	public static int nextId = 0;
	public static int nextNeuronId = 0; 
	public static final ArrayList<NodeMutation> nodeMutations = new ArrayList<>();
	public static final ArrayList<Edge> connMutations = new ArrayList<>();
	
	public static final float C1 = 1;
	public static final float C2 = 1;
	public static final float C3 = 0.4f;

	@Override
	public BehaviourGenotype treeGenotypeOnePointCrossover(TreeGenotype child, TreeGenotype parent1, TreeGenotype parent2, int splitIndex) {
		return null;
	}

	@Override
	public BehaviourGenotype treeGenotypeUniformCrossover(TreeGenotype child, TreeGenotype parent1, TreeGenotype parent2) {
		return null;
	}

	
	
	public class NodeMutation implements Serializable{
		
		public int connectionId;
		public int newNodeId;
		public int elementId_1;
		public int elementId_2;
		
		@Override
		public boolean equals(Object o) {
			NodeMutation n = (NodeMutation) o;
			return this.connectionId == n.connectionId;
		}
	}
	
	public enum NodeType {
		INPUT, OUTPUT, HIDDEN
	}
	
	public class Node implements Serializable{
		
		public final int id;
		final NodeType type;
		
		public Node(int id, NodeType type) {
			this.id = id;
			this.type = type;
		}
		
		public Node(Node orig){
			this.id = orig.id;
			this.type = orig.type;
		}
		
		@Override
		public boolean equals(Object o) {
			Node n = (Node) o;
			return this.id == n.id;
		}
	}
	
	public class Edge implements Comparable<Edge>, Serializable{
		public int innov;
		public final int fromNeuron;
		public final int toNeuron;
		public float weight;
		public boolean disabled = false;
		public boolean isRecurrent = false;
		
		public Edge(boolean incremental, int fromNeuron, int toNeuron) {
			this.fromNeuron = fromNeuron;
			this.toNeuron = toNeuron;
			this.weight = Utils.randomFloat(-10, 10);
			if (incremental) {
				synchronized(LOCK){
					this.innov = nextId++;
				}
			}
			else {
				this.innov = -1;
			}
		}
		
		public Edge(Edge orig) {
			this.innov = orig.innov;
			this.fromNeuron = orig.fromNeuron;
			this.toNeuron = orig.toNeuron;
			this.disabled = orig.disabled;
			this.weight = orig.weight;
		}
		
		@Override
		public boolean equals(Object o) {
			Edge other = (Edge) o;
			return this.innov == other.innov || (this.fromNeuron == other.fromNeuron && this.toNeuron == other.toNeuron);
		}

		@Override
		public int compareTo(Edge e) {
			if		(this.innov < e.innov)		return -1;
			else if (this.innov > e.innov)		return 1;
			return 0;
		}
	
	}
	
	public final ArrayList<Node> nodes;
	public final ArrayList<Edge> edges;
	public ArrayList<Edge> possibleEdges;
	
	public NeatGenotype() {
		this.nodes = new ArrayList<>();
		this.edges = new ArrayList<>();
		this.possibleEdges = new ArrayList<>();
	}
	public NeatGenotype(int inputSize, int outputSize) {
		this();
		for (int i = 0; i < inputSize; i++) {
			this.nodes.add(new Node(nextNeuronId++, NodeType.INPUT));
		}
		for (int i = 0; i < outputSize; i++) {
			this.nodes.add(new Node(nextNeuronId++, NodeType.OUTPUT));
		}
		
		for(int i = 0; i < inputSize; i++) {
			for (int o = inputSize; o < inputSize+outputSize; o++) {
				this.edges.add(new Edge(true, this.nodes.get(i).id, this.nodes.get(o).id));
			}
		}
	}
	
	public NeatGenotype(NeatGenotype orig) {
		this();
		for (Edge e : orig.edges) {
			this.edges.add(new Edge(e));
		}
		for (Node n : orig.nodes) {
			this.nodes.add(new Node(n));
		}
	}

	@Override
	public Genotype copy() {
		return new NeatGenotype(this);
	}
	
//	public ArrayList<Edge> getPossibleConnections() {
//		ArrayList<Edge> possibleConnections = new ArrayList<>();
//		
//		for (Node fromNode : this.nodes) {
//			for (Node toNode : this.nodes) {
//				if (toNode.type != NodeType.INPUT) {
//					Edge edge = new Edge(false, fromNode.id, toNode.id);
//					if (!this.edges.contains(edge) && CreativeBehaviour.neatCfg.getBoolean("recurrent")){
//						if (fromNode.type == NodeType.OUTPUT || fromNode.equals(toNode)) { 
//							edge.isRecurrent = true;
//							possibleConnections.add(edge);
//							continue;
//						}
//						ArrayDeque<Edge> queue = new ArrayDeque<>();
//						queue.add(edge);
//						while (!queue.isEmpty()) {
//							Edge current = queue.poll();
//							if (current.toNeuron == edge.fromNeuron) {
//								edge.isRecurrent = true;
//								possibleConnections.add(edge);
//								break;
//							}
//							for (Edge e : this.edges) {
//								if (current.toNeuron == e.fromNeuron && !e.isRecurrent) {
//									queue.add(e);
//								}
//							}
//						}
//					} 
//					else if (!this.edges.contains(edge)) {
//						if (fromNode.type == NodeType.OUTPUT || fromNode.equals(toNode)) { 
//							continue;
//						}
//						ArrayDeque<Edge> queue = new ArrayDeque<>();
//						queue.add(edge);
//						while (!queue.isEmpty()) {
//							Edge current = queue.poll();
//							if (current.toNeuron == edge.fromNeuron) {
//								edge.isRecurrent = true;
//								break;
//							}
//							for (Edge e : this.edges) {
//								if (current.toNeuron == e.fromNeuron && !e.isRecurrent) {
//									queue.add(e);
//								}
//							}
//						}
//						if (!edge.isRecurrent) possibleConnections.add(edge);
//					}
//				}
//			}
//		}
//		
//		return possibleConnections; 
//	}
	
	public ArrayList<Edge> getPossibleConnections() {
		ArrayList<Edge> possibleConnections = new ArrayList<>();
		
		for (Node fromNode : this.nodes) {
			for (Node toNode : this.nodes) { 
				if (!fromNode.equals(toNode)) {
					if  (fromNode.type == NodeType.INPUT) {
						Edge e = new Edge(false, fromNode.id, toNode.id);
						if (!this.edges.contains(e)) {
							possibleConnections.add(e);
						}
					}
					else if (fromNode.type == NodeType.HIDDEN && toNode.type == NodeType.OUTPUT) {
						Edge e = new Edge(false, fromNode.id, toNode.id);
						if (!this.edges.contains(e)) {
							possibleConnections.add(e);
						}
					}
				}
			}
		}
		
		return possibleConnections;
		
	}

	@Override
	public Genotype crossover(Genotype g, Crossover type, int isBestParent) {
		NeatGenotype child = new NeatGenotype();
		NeatGenotype other = (NeatGenotype) g;
		
		if (isBestParent == 1) {
			for (Edge e : this.edges) {
				Node toNode = null;
				Node fromNode = null;
				for (Node n : this.nodes) {
					if (n.id == e.toNeuron) toNode = n;
					if (n.id == e.fromNeuron) fromNode = n;
				}
				if (!child.nodes.contains(toNode)) child.nodes.add(toNode);
				if (!child.nodes.contains(fromNode)) child.nodes.add(fromNode);
				if (other.edges.contains(e)) {
					if (Utils.randomFloat() < 0.5) {
						child.edges.add(new Edge(e));
					}
					else {
						child.edges.add(new Edge(other.edges.get(other.edges.indexOf(e))));
					}
				}
				else {
					child.edges.add(new Edge(e));
				}
			}
		}
		else if (isBestParent == -1) {
			for (Edge e : other.edges) {
				Node toNode = null;
				Node fromNode = null;
				for (Node n : other.nodes) {
					if (n.id == e.toNeuron) toNode = n;
					if (n.id == e.fromNeuron) fromNode = n;
				}
				if (!child.nodes.contains(toNode)) child.nodes.add(toNode);
				if (!child.nodes.contains(fromNode)) child.nodes.add(fromNode);
				if (this.edges.contains(e)) {
					if (Utils.randomFloat() < 0.5) {
						child.edges.add(new Edge(e));
					}
					else {
						child.edges.add(new Edge(this.edges.get(this.edges.indexOf(e))));
					}
				}
				else {
					child.edges.add(new Edge(e));
				}
			}
		}
		else {
			for (Edge e : this.edges) {
				Node toNode = null;
				Node fromNode = null;
				for (Node n : this.nodes) {
					if (n.id == e.toNeuron) toNode = n;
					if (n.id == e.fromNeuron) fromNode = n;
				}
				if (!child.nodes.contains(toNode)) child.nodes.add(toNode);
				if (!child.nodes.contains(fromNode)) child.nodes.add(fromNode);
				if (other.edges.contains(e)) {
					if (Utils.randomFloat() < 0.5) {
						child.edges.add(new Edge(e));
					}
					else {
						child.edges.add(new Edge(other.edges.get(other.edges.indexOf(e))));
					}
				}
				else {
					child.edges.add(new Edge(e));
				}
			}
			for (Edge e : other.edges) {
				if (!child.edges.contains(e)) {
					Node toNode = null;
					Node fromNode = null;
					for (Node n : other.nodes) {
						if (n.id == e.toNeuron) toNode = n;
						if (n.id == e.fromNeuron) fromNode = n;
					}
					if (!child.nodes.contains(toNode)) child.nodes.add(toNode);
					if (!child.nodes.contains(fromNode)) child.nodes.add(fromNode);
					
					child.edges.add(new Edge(e));
				}
			}
		}
		
		return child;
	}
	
	@Override
	public Genotype crossover(Genotype g, Crossover type) {
		return this.crossover(g, type, 0);
	}
	
	@Override
	public void mutate(double prob) {
		
		// Mutate add node
		if (Utils.randomDouble() < Double.valueOf(CreativeBehaviour.neatCfg.get("node_mutation_rate"))) { // TODO better condition
			int disabledCount = 0;
			for (Edge e : this.edges) {
				if (e.disabled) disabledCount++;
			}
			if (disabledCount != this.edges.size()) {
				int index = Utils.randomInt(edges.size()-disabledCount);
				
				int indexCount = 0;
				for (int i = 0; i <= index; i++) {
					if (this.edges.get(i).disabled) index++;
				}
				
				NodeMutation mutation = new NodeMutation();
				Edge fromOld;
				Edge fromNew;
				mutation.connectionId = index;
				mutation.connectionId = edges.get(index).innov;

				if (nodeMutations.contains(mutation)) {
					mutation = nodeMutations.get(nodeMutations.indexOf(mutation));
					fromOld = new Edge(false, edges.get(index).fromNeuron, mutation.newNodeId);
					fromOld.innov = mutation.elementId_1;

					fromNew = new Edge(false, mutation.newNodeId, edges.get(index).toNeuron);
					fromNew.innov = mutation.elementId_2;
					fromNew.weight = edges.get(index).weight;

				}
				else {
					synchronized(LOCK) {
						mutation.newNodeId = nextNeuronId++;
					}
					fromOld = new Edge(true, edges.get(index).fromNeuron, mutation.newNodeId);
					mutation.elementId_1 = fromOld.innov;
					fromNew = new Edge(true, mutation.newNodeId, edges.get(index).toNeuron);
					fromNew.weight = edges.get(index).weight;
					mutation.elementId_2 = fromNew.innov;

					nodeMutations.add(mutation);
				}
				this.edges.get(index).disabled = true;
				this.edges.add(fromOld);
				this.edges.add(fromNew);

				this.nodes.add(new Node(mutation.newNodeId, NodeType.HIDDEN));
			}
		}
		// Mutate add connection
		if (Utils.randomDouble() < Double.valueOf(CreativeBehaviour.neatCfg.get("connection_mutation_rate"))) { 
			possibleEdges = getPossibleConnections();
			if (!possibleEdges.isEmpty()) {
				Edge conn = possibleEdges.get(Utils.randomInt(possibleEdges.size()));
				if (connMutations.contains(conn)) {
					Edge c = connMutations.get(connMutations.indexOf(conn));
					conn.innov = c.innov;
					conn.weight = c.weight;
				}
				else {
					synchronized(LOCK) {
						conn.innov = nextId++;
					}
					connMutations.add(conn);
				}
				conn.weight = Utils.randomFloat(-10, 10);
				this.edges.add(conn);
			}
		}
		
		// Weight Mutation
		for (Edge conn : this.edges) {
			if (Utils.randomDouble() < prob) {
				if (Utils.randomDouble() < 0.9) {
					conn.weight += Utils.randomGausian(0.0, 0.1); 
				}
				else {
					conn.weight = Utils.randomFloat(-10, 10);
				}
			}
		}
		
	}
	
	public double distanceTo(NeatGenotype n) {
		int excessCount = 0;
		int disjointCount = 0;
		double weightsAvg = 0;
		int weightsCount = 0;
		
		int thisFirst = this.edges.get(0).innov;
		int thisLast = this.edges.get(this.edges.size()-1).innov;
		
		int otherFirst = n.edges.get(0).innov;
		int otherLast = n.edges.get(n.edges.size()-1).innov;
		
		for (int i = 0; i < this.edges.size(); i++) {
			// Is Excess
			if (this.edges.get(i).innov < otherFirst){
				excessCount++;
			}
			// Is Matching or Disjoint
			else if (this.edges.get(i).innov <= otherLast) {
				if (n.edges.contains(this.edges.get(i))) {
					double thisWeight = this.edges.get(i).weight;
					double otherWeight = n.edges.get(n.edges.indexOf(this.edges.get(i))).weight;
					
					weightsAvg += Math.abs(thisWeight-otherWeight);
					weightsCount++;
				}
				else {
					disjointCount++;
				}
			}
			// Is Excess
			else {
				excessCount++;
			}
		}
		
		for (int i = 0; i < n.edges.size(); i++) {
			// Is Excess
			if (n.edges.get(i).innov < thisFirst && !this.edges.contains(n.edges.get(i))){
				excessCount++;
			}
			// Is Disjoint
			else if (n.edges.get(i).innov <= otherLast && !this.edges.contains(n.edges.get(i))) {
				disjointCount++;
			}
			// Is Excess
			else if (!this.edges.contains(n.edges.get(i))){
				excessCount++;
			}
		}
		if (weightsCount != 0) {
			weightsAvg /= (double) weightsCount;
		}
		
		
		int N = 0;
		if (this.edges.size() < 20 && n.edges.size() < 20) {
			N = 20;
		}
		else {
			N = Math.max(this.edges.size(), n.edges.size());
		}
		return C1*excessCount/(double)N + C2*disjointCount/(double)N + C3*weightsAvg;
	}
	
}
