package creativebehaviour.models.physical;

import creativebehaviour.ents.Body;
import creativebehaviour.ents.Body.BodyWrapper;
import creativebehaviour.ents.misc.EntType;
import creativebehaviour.ents.wrapper.NullWrapper;
import creativebehaviour.ents.wrapper.Wrapper;
import creativebehaviour.models.BehaviourGenotype;
import creativebehaviour.models.Creature;
import creativebehaviour.util.tree.Node;
import creativebehaviour.util.tree.Tree;
import creativebehaviour.util.Utils;
import creativebehaviour.util.io.CrebeIO;
import eval.models.Crossover;
import eval.models.Genotype;
import eval.util.Config;
import java.awt.Color;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Stack;

/**
 *
 * @author Endre
 */
public class TreeGenotype extends Genotype {

	private static double ent_remove_mp;
	private static double ent_changetype_mp;
	private static double ent_add_mp;
	private static double mutation_factor;
	
	public final static long serialVersionUID = 3824693946096405683l;
	
	public Tree<Wrapper> tree;
	public Color color;
	public BehaviourGenotype behaviourGenotype;
	
	
	public TreeGenotype() {
		tree = new Tree<>();
		color = new Color(Utils.randomInt());
		addEntityWrapperToNode(tree.root, new Body.BodyWrapper());
	}
	
	
	/**
	 * Copy constructor.
	 *
	 * @param orig
	 */
	public TreeGenotype(TreeGenotype orig) {
		tree = new Tree<>(orig.tree);
		color = new Color(orig.color.getRGB());
		behaviourGenotype = orig.behaviourGenotype;
	}
	
	
	public void setBehaviourGenotype(BehaviourGenotype g) {
		this.behaviourGenotype = g;
	}
	
	
	public Node<Wrapper> rootNode() {
		return tree.root;
	}
	

	public Wrapper rootEnt() {
		return tree.root.obj;
	}
	

	public boolean containsNode(Node<Wrapper> node) {
		for (Node<Wrapper> node2 : tree.nodeIterator()) {
			if (node == node2) {
				return true;
			}
		}
		return false;
	}

	
	public final void addEntityWrapperToNode(Node<Wrapper> node, Wrapper wrapper) {
		assert this.containsNode(node);

		if (node.obj == null || (node.obj instanceof NullWrapper)) {
			node.obj = wrapper;
			for (int i = 0; i < wrapper.ENT_ATTACH_POINTS; i++) {
				node.addChildObj(null);
			}
		} else {
			throw new RuntimeException("No free attach point.");
		}
	}

	
	@Override
	public Genotype copy() {
		return new TreeGenotype(this);
	}

	
	@Override
	public Genotype crossover(Genotype g, Crossover type) {
		TreeGenotype tg = (TreeGenotype) g;
		switch (type) {
			case ONE_POINT:
				return onePointCrossover(tg);
			case UNIFORM:
				return uniformCrossover(tg);
			default:
				throw new AssertionError(type.name());
		}
	}

	
	@Override
	public void mutate(double prob) {
		if (Utils.randomEvent(prob))
			mutateRecursive(tree.root, Utils.randomGausian(0, mutation_factor));
	}
	

	private void mutateRecursive(Node<Wrapper> node, double mutateFactor) {
		if (node.obj != null)
			node.obj.mutate(mutateFactor);

		for (Node<Wrapper> child : node) {
			boolean childIsNull = (child.obj == null) || (child.obj instanceof NullWrapper);
			
			if (!(child.obj instanceof BodyWrapper)) {
				if (childIsNull) {
					if (Utils.randomEvent(ent_add_mp)) { // Add new ent to empty attach point
						Wrapper w = createRandomEnt().createWrapper();
						addEntityWrapperToNode(child, w);
					}
				}
				else {
					double p = Utils.randomDouble();
					if (p < ent_remove_mp) { // Remove ent
						child.obj = null;
						child.children.clear();
					} else if (p < (ent_remove_mp + ent_changetype_mp)) { // Switch ent
						Wrapper w = createRandomEnt().createWrapper();
						child.obj = null;
						child.children.clear();
						addEntityWrapperToNode(child, w);
					}
				}
			}

			mutateRecursive(child, mutateFactor);
		}
	}
	
	
	private EntType createRandomEnt() {
		EntType type;
		
		do {
			type = EntType.values()[Utils.randomInt(EntType.ENT_COUNT)];
		} while (type == EntType.BODY);
		
		return type;
	}

	
	private TreeGenotype onePointCrossover(TreeGenotype tg) {
		assert this.rootEnt().ENT_ATTACH_POINTS == tg.rootEnt().ENT_ATTACH_POINTS;

		int size = this.rootEnt().ENT_ATTACH_POINTS;
		int splitPoint = Utils.randomInt(size + 1);
		float hFactor = splitPoint / (float) size;

		BodyWrapper childBody = new BodyWrapper();
		BodyWrapper thisBody = (BodyWrapper) this.rootEnt();
		BodyWrapper gBody = (BodyWrapper) tg.rootEnt();

		// Calculate and set radius
		childBody.radius = thisBody.radius * hFactor + gBody.radius * (1 - hFactor);

		// Calculate and set colors
		int red = 0xFF & (int) (color.getRed() * hFactor + tg.color.getRed() * (1 - hFactor));
		int green = 0xFF & (int) (color.getGreen() * hFactor + tg.color.getGreen() * (1 - hFactor));
		int blue = 0xFF & (int) (color.getBlue() * hFactor + tg.color.getBlue() * (1 - hFactor));

		TreeGenotype child = new TreeGenotype();
		child.tree.root.obj = childBody;
		child.color = new Color(red, green, blue);

		for (int i = 0; i < splitPoint; i++) {
			child.rootNode().setChildNode(new Node<Wrapper>(this.rootNode().getChild(i), false), i);
		}
		
		for (int i = splitPoint; i < size; i++) {
			child.rootNode().setChildNode(new Node<Wrapper>(tg.rootNode().getChild(i), false), i);
		}

		child.behaviourGenotype = this.behaviourGenotype.treeGenotypeOnePointCrossover(child, this, tg, splitPoint);
		
		return child;
	}
	

	public TreeGenotype uniformCrossover(TreeGenotype tg) {
		assert this.rootEnt().ENT_ATTACH_POINTS == tg.rootEnt().ENT_ATTACH_POINTS;
		float hFactor = 0.5f;
		BodyWrapper childBody = new BodyWrapper();
		BodyWrapper thisBody = (BodyWrapper) this.rootEnt();
		BodyWrapper gBody = (BodyWrapper) tg.rootEnt();

		// Calculate and set radius
		childBody.radius = thisBody.radius * hFactor + gBody.radius * (1 - hFactor);

		// Calculate and set colors
		int red = 0xFF & (int) (color.getRed() * hFactor + tg.color.getRed() * (1 - hFactor));
		int green = 0xFF & (int) (color.getGreen() * hFactor + tg.color.getGreen() * (1 - hFactor));
		int blue = 0xFF & (int) (color.getBlue() * hFactor + tg.color.getBlue() * (1 - hFactor));

		TreeGenotype child = new TreeGenotype();
		child.tree.root.obj = childBody;
		child.color = new Color(red, green, blue);

		for (int i = 0; i < this.rootEnt().ENT_ATTACH_POINTS; i++) {
			if (Utils.randomDouble() < 0.5) {
				child.rootNode().setChildNode(new Node<>(this.rootNode().getChild(i), false), i);
			} else {
				child.rootNode().setChildNode(new Node<>(tg.rootNode().getChild(i), false), i);
			}
		}
		
		return child;
	}
	
	
	/**
	 * Returns true if this tree genotype id identical (entity wrapper wise)
	 * to the supplied tree genotype. The two genotypes must have the same entity types
	 * at the same position in order to be wrapper equal.
	 * @param other
	 * @return 
	 */
	public boolean wrapperEqual(TreeGenotype other) {
		if (this == other) return true;
		if (other == null) return false;
		
		Iterator<Wrapper> it1 = tree.iterator();
		Iterator<Wrapper> it2 = other.tree.iterator();
		
		while (it1.hasNext() && it2.hasNext()) {
			Wrapper w1 = it1.next();
			Wrapper w2 = it2.next();
			if (w1.entType != w2.entType) return false;
		}
		
		return !(it1.hasNext() || it2.hasNext());
	}
	
	public static TreeGenotype readFromSCDFile(String filename) throws IOException{
		try {
			File f = new File(filename);
			Creature c = CrebeIO.importCreature(f);
			return c.treeGenotype;
		} catch (ClassNotFoundException e) {
			System.out.println(e.getMessage());
			return null;
		}
	}
	
	public static TreeGenotype readFromTxtFile(String filename) throws IOException, EntityParseException {
		try {
			BufferedReader br = new BufferedReader(new FileReader(filename));
			
			ArrayList<String> lines = new ArrayList<>();
			String l = br.readLine();
			while (l != null) {
				lines.add(l);
				l = br.readLine();
			}
			
			for (int i = lines.size()-1; i >= 0; i--) {
				if (lines.get(i).matches("\t*-")) lines.remove(i);
				else break;
			}
			
			Stack<Node<Wrapper>> stack = new Stack<>();
			Node<Wrapper> current = null;
			int lineCount = 0;
			
			for (String line : lines) {
				lineCount++;

				if ("".equals(line)) {
					if (stack.isEmpty()) {
						continue;
					} else {
						break;
					}
				}

				int indentation = 0;
				try {
					while (line.charAt(indentation) == '\t') {
						indentation++;
					}
				} catch (StringIndexOutOfBoundsException e) {
					throw new EntityParseException(String.format("No entity specified on line %d.", lineCount));
				}

				int deltaIndent = indentation - stack.size();

				if (deltaIndent == 1) {
					if (current == null) {
						throw new EntityParseException(String.format("Null-entity used as parent on line %d.", lineCount));
					}
					stack.push(current);
				} else if (deltaIndent > 1) {
					throw new EntityParseException(String.format("Parent skipped on line %d.", lineCount));
				} else {
					for (int i = deltaIndent; i < 0; i++) {
						stack.pop();
					}
				}

				String[] lineSplit = line.substring(indentation).split(",");
				String entStr = lineSplit[0];

				if (!"-".equals(entStr)) {
					try {
						EntType entt = EntType.valueOf(entStr.toUpperCase());
						Wrapper entw = entt.createWrapper();
						int paramIndex = 1;
						
						Field[] entFields = entw.getClass().getDeclaredFields();
						for (Field field : entFields) {
							if (paramIndex >= lineSplit.length) break;
							
							if (!Modifier.isStatic(field.getModifiers())) {
								String param = lineSplit[paramIndex++];
								Object parsedParam = null;
								Class c = field.getType();
								
								if (c == float.class)        parsedParam = Float.parseFloat(param);
								else if (c == double.class)  parsedParam = Double.parseDouble(param);
								else if (c == byte.class)    parsedParam = Byte.parseByte(param);
								else if (c == short.class)   parsedParam = Short.parseShort(param);
								else if (c == int.class)     parsedParam = Integer.parseInt(param);
								else if (c == long.class)    parsedParam = Long.parseLong(param);
								else if (c == boolean.class) parsedParam = Boolean.parseBoolean(param);
								else if (c == char.class)    parsedParam = param.charAt(0);
								else if (c == String.class)  parsedParam = param;
								
								try {
									field.set(entw, parsedParam);
								} catch (IllegalAccessException | IllegalArgumentException e) {
									break;
								}
							}
						}
						
						current = new Node<>(entw);
					} catch (IllegalArgumentException e) {
						throw new EntityParseException(String.format("Unknown entity \"%s\" on line %d.", entStr.toUpperCase(), lineCount));
					}

					if (stack.isEmpty()) {
						stack.push(current);
					} else {
						stack.peek().addChildNode(current);
					}
				} else {
					if (stack.isEmpty()) {
						throw new EntityParseException(String.format("Null-entity used as parent on line %d.", lineCount));
					} else {
						current = new Node<>(new NullWrapper());
						stack.peek().addChildNode(current);
					}
					current = null;
				}
			}

			Tree<Wrapper> tree = new Tree<>();
			while (!stack.isEmpty()) {
				tree.root = stack.pop();
			}

			if (!(tree.root.obj instanceof BodyWrapper)) throw new EntityParseException("First entity must be a Body entity.");
			TreeGenotype treeg = new TreeGenotype();
			treeg.tree = tree;

			return treeg;
		} catch (EntityParseException e) {
			throw new EntityParseException("An exception occurred while parsing the entity file: \"" + filename + "\".", e);
		}
	}
	
	
	public static void parseMpConfig(Config cfg) {
		ent_remove_mp = cfg.getDouble("ent_remove_mp");
		ent_changetype_mp = cfg.getDouble("ent_changetype_mp");
		ent_add_mp = cfg.getDouble("ent_add_mp");
		
		if (ent_remove_mp + ent_changetype_mp > 1.0) {
			System.err.println("WARNING! ent_remove_mp + ent_changetype_mp exceeds 1.0!");
			try {
				Thread.sleep(2000);
			} catch (Exception e) {
			}
		}
		
		mutation_factor = cfg.getDouble("mutation_factor");
	}
	
	
}
