package creativebehaviour.simulation;

import creativebehaviour.ents.Entity;
import creativebehaviour.ents.Spike;
import creativebehaviour.ents.misc.AttachPoint;
import creativebehaviour.models.BenchmarkSimpleCreature;
import creativebehaviour.models.BenchmarkMovingCreature;
import creativebehaviour.models.Creature;
import creativebehaviour.util.Pointf;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.NoninvertibleTransformException;
import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.Stack;
import creativebehaviour.CreativeBehaviour;
import creativebehaviour.models.BenchmarkFightingCreature;
import creativebehaviour.models.physical.EntityParseException;

/**
 *
 * @author Endre
 */
public class ArenaWorld extends World {
	
	public final int BENCHMARK_CREATURE = 0;
	public final int TESTED_CREATURE = 1;
	public final int NR_OF_TICKS = 3600;
	
	private final int MAX_ENERGY = Integer.parseInt(CreativeBehaviour.crebeCfg.get("start_energy"));
	
	public ArenaWorld(int width, int height, WorldType type) {
		super(width, height, type, true, false);
		try {
			switch (this.worldType){
				case ARENA_1: this.addCreature(new BenchmarkSimpleCreature()); break;
				case ARENA_2: this.addCreature(new BenchmarkMovingCreature()); break;
				case ARENA_WORLD: this.addCreature(new BenchmarkFightingCreature()); break;
			}
			this.creatures.get(BENCHMARK_CREATURE).position.x = 100;
			this.creatures.get(BENCHMARK_CREATURE).position.y = 300;
		} catch (IOException | EntityParseException e ) {
			throw new RuntimeException(e);
		} 
	}
	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		g.setColor(Color.BLACK);
		g.drawRect(0, 0, WIDTH, HEIGHT);
		
		// TODO colorize creature
		synchronized (creatures) {
			int i = 0;
			for (Creature c : creatures) {
				g.setColor(Color.BLACK);
				c.render(g);
				i++;
			}
		}
		
	}
	@Override
	public void setFitnessOnCreatures() {
		double benchmarkHealthRatio = creatures.get(BENCHMARK_CREATURE).getEnergy() / (double) MAX_ENERGY;
		if(benchmarkHealthRatio < 0) benchmarkHealthRatio = 0;
		double testedHealthRatio = creatures.get(TESTED_CREATURE).getEnergy() / (double) MAX_ENERGY;
		if(testedHealthRatio < 0) testedHealthRatio = 0;
		double timeRatio = creatures.get(TESTED_CREATURE).getTicksLived() / CreativeBehaviour.crebeCfg.getDouble("simulation_ticks");
		
		
		/*if (benchmarkHealthRatio <= testedHealthRatio) {
			creatures.get(BENCHMARK_CREATURE).setFitness(((benchmarkHealthRatio + (1-testedHealthRatio) + timeRatio) / (double) 3));
			creatures.get(TESTED_CREATURE).setFitness(((testedHealthRatio + (1-benchmarkHealthRatio) + (1-timeRatio)) / (double) 3));
			
		}
		else {
			creatures.get(BENCHMARK_CREATURE).setFitness(((benchmarkHealthRatio + (1-testedHealthRatio)) / (double) 2));
			creatures.get(TESTED_CREATURE).setFitness(((testedHealthRatio + (1-benchmarkHealthRatio)) / (double) 2));
		}*/
		double benchmarkHealth = Math.max(creatures.get(BENCHMARK_CREATURE).getEnergy(), 0);
		double testedHealth = Math.max(creatures.get(TESTED_CREATURE).getEnergy(), 0);
		creatures.get(BENCHMARK_CREATURE).setFitness((benchmarkHealth-testedHealth-creatures.get(BENCHMARK_CREATURE).getTicksLived())/100);
		creatures.get(TESTED_CREATURE).setFitness((testedHealth-benchmarkHealth-creatures.get(TESTED_CREATURE).getTicksLived())/100);
	}
	
	@Override
	public boolean continueCheck(){
		// Check if a creature is dead
		return creatures.get(BENCHMARK_CREATURE).isAlive() && creatures.get(TESTED_CREATURE).isAlive();
	}
	
	@Override
	protected void customWorldTick(int tickCounter) {
		synchronized (creatures) {
			if (creatures.size() >= 2) {
				// Tested creature
				Entity current;
				Stack<Entity> searchStack = new Stack<>(); // TODO optimize to field
				searchStack.add(creatures.get(TESTED_CREATURE).rootEnt);
				while (!searchStack.isEmpty()){
					current = searchStack.pop();
					for (AttachPoint ap : current.getAttachPoints()) {
						if (ap.getChild() != null) searchStack.add(ap.getChild());
					}
					if (current instanceof Spike) {
						Spike spike = (Spike) current;		
						Pointf pos = spike.getPointGlobalPos();
						
						spike.setSpeed((int) (Math.abs(pos.x-spike.prevPos.x)), (int) (Math.abs(pos.y-spike.prevPos.y)));
						
						spike.prevPos = pos;

						if (Math.abs(pos.distanceTo(creatures.get(BENCHMARK_CREATURE).position)) < creatures.get(BENCHMARK_CREATURE).rootEnt.getRadius()) {
							creatures.get(BENCHMARK_CREATURE).takeDamage(spike.getDamage());
						}

					}

				}
				
				searchStack.clear();
				searchStack.add(creatures.get(BENCHMARK_CREATURE).rootEnt);
				while (!searchStack.isEmpty()){
					current = searchStack.pop();
					for (AttachPoint ap : current.getAttachPoints()) {
						if (ap.getChild() != null) searchStack.add(ap.getChild());
					}
					if (current instanceof Spike) {
						Spike spike = (Spike) current;		
						Pointf pos = spike.getPointGlobalPos();
						
						spike.setSpeed((int) (pos.x-spike.prevPos.x), (int) (pos.y-spike.prevPos.y));
						
						spike.prevPos = pos;

						if (Math.abs(pos.distanceTo(creatures.get(TESTED_CREATURE).position)) < creatures.get(TESTED_CREATURE).rootEnt.getRadius()) {
							creatures.get(TESTED_CREATURE).takeDamage(spike.getDamage());
						}

					}

				}
			}
			
		}
	}

	@Override
	protected void customWorldRender(Graphics2D g) throws NoninvertibleTransformException {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}
	
	@Override
	public void addCreature(Creature c){
		super.addCreature(c);
		c.position.x = 500;
		c.position.y = 300;
		c.rotation = 270;
	}

	@Override
	protected Iterator customWorldCollectionIterator() {
		return null;
	}

	@Override
	protected int getNumCustomEnts() {
		return 0;
	}
}
