package creativebehaviour.simulation;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.graphics.Render;
import creativebehaviour.util.Pointf;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;
import java.awt.geom.NoninvertibleTransformException;

/**
 *
 * @author Endre
 */
public class Food implements WorldEntity {
	public static final int DEFAULT_NOURISHMENT = CreativeBehaviour.crebeCfg.getInt("food_nourishment");
	
	public float nourishment;
	public final Pointf position;
	private static final Stroke STROKE = new BasicStroke(2);
	private final Shape shape;
	private final int size = 5;
	
	
	public Food(float nourishment) {
		this.nourishment = nourishment;
		position = new Pointf();
		shape = new Ellipse2D.Double(-size, -size, size*2, size*2);
	}

	
	public Food() {
		this(DEFAULT_NOURISHMENT);
	}

	
	@Override
	public void render(Graphics2D g) throws NoninvertibleTransformException {
		Color color;
		
		if (this.nourishment > 0) color = Color.RED;
		else color = Color.GRAY;
		
		g.translate(position.x, position.y);
		Render.renderOutlinedShape(shape, Color.BLACK, color, STROKE, g);
		g.translate(-position.x, -position.y);
	}

	
	@Override
	public String toString() {
		return "Food nourishment: " + nourishment;
	}

	
	@Override
	public Pointf getPos() {
		return position;
	}

	
	@Override
	public void tick(World world) {
	}
	

	@Override
	public Type getWorldEntType() {
		return Type.FOOD;
	}
	
	
	
}
