package creativebehaviour.simulation;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.ents.Mouth;
import creativebehaviour.ents.misc.InputEntity;
import creativebehaviour.models.Creature;
import creativebehaviour.util.Pointf;
import creativebehaviour.util.Utils;
import java.awt.Graphics2D;
import java.awt.geom.NoninvertibleTransformException;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author Endre
 */
public class FoodWorld extends World {
	public static final double MOUTH_EAT_RADIUS = 30;
	public static final int FOOD_SPAWNRATE = CreativeBehaviour.crebeCfg.getInt("food_spawnrate");
	
	private final ArrayList<Food> food;
	
	
	public FoodWorld(int width, int height) {
		super(width, height, WorldType.FOOD_WORLD);
		food = new ArrayList<>();
	}
	

	@Override
	protected void customWorldTick(int tickCounter) {
		//ArrayList<Creature> creaturesCopy = getCreatures();
		
		for (Creature c : creatures) {
			c.inputEntities.stream().filter(e -> e instanceof Mouth).forEach(e -> {
				Mouth mouth = (Mouth) e;		
				Iterator<Food> it = food.iterator();
				Pointf pos = mouth.getGlobalPos();

				while (it.hasNext()) {
					Food food = it.next();
					if (Math.abs(pos.distanceTo(food.position)) < MOUTH_EAT_RADIUS) {
						c.eatFood(food);
						it.remove();
					}
				}
			});
		}
		
		if (FOOD_SPAWNRATE > 0 && tickCounter % FOOD_SPAWNRATE == 0) {
			spawnFood(Utils.randomDouble()*WIDTH, Utils.randomDouble()*HEIGHT);
		}
	}
	
	
	public void spawnFood(double x, double y) {
		Food f = new Food();
		f.position.x = x;
		f.position.y = y;
		addFood(f);
	}
	
	
	public void addFood(Food f) {
		synchronized (WORLD_ENTS_LOCK) {
			food.add(f);
		}
	}
	

	@Override
	protected void customWorldRender(Graphics2D g) throws NoninvertibleTransformException {
		for (int i = 0; i < food.size(); i++) {
			food.get(i).render(g);
		}
	}
	
	public ArrayList<Food> getFood() {
		return food;
	}

	
	@Override
	protected Iterator customWorldCollectionIterator() {
		return food.iterator();
	}

	
	@Override
	protected int getNumCustomEnts() {
		return food.size();
	}

	
	@Override
	public void setFitnessOnCreatures() {
		for (Creature c : this.creatures) {
			c.setFitness(c.getFoodEaten());
		}
	}
}
