package creativebehaviour.simulation;

import creativebehaviour.models.Creature;


/**
 *
 * @author Endre
 */
public class Simulation {
	
	private World world;
	private final int totalTicks;
	
	
	public Simulation(WorldType type, int totalTicks) {
		switch(type) {
			case STANDARD:
				world = WorldFactory.makeStandardWorld();
				break;
			case FOOD_WORLD:
				world = WorldFactory.makeFoodWorld();
				break;
			case POISON_WORLD:
				world = WorldFactory.makeFoodAndPoisonWorld();
				break;
			case ARENA_1:
				world = WorldFactory.makeArenaTrainingWorld(type);
				break;
			case ARENA_2:
				world = WorldFactory.makeArenaTrainingWorld(type);
				break;
			case ARENA_WORLD:
				world = WorldFactory.makeArenaTrainingWorld(type);
				break;
			case MULTI_AGENT:
				throw new UnsupportedOperationException("Multi agent world");
			case FOOD_WORLD_EDGE_SPAWN:
				world = WorldFactory.makeFoodWorldEdgeSpawn();
				break;
			default:
				throw new AssertionError(type.name());	
		}
		this.totalTicks = totalTicks;
	}
	
	
	public void runSimulation() {
		for (int i = 0; i < totalTicks && world.continueCheck(); i++) {
			world.tick();
		}
		world.setFitnessOnCreatures();
	}
	
	
	public void addCreature(Creature c) {
		this.world.addCreature(c);
	}
	
	
	public World getWorld() {
		return world;
	}
	
}
