package creativebehaviour.simulation;

import creativebehaviour.CreativeBehaviour;
import creativebehaviour.util.Pointf;
import creativebehaviour.util.Utils;

/**
 *
 * @author Endre
 */
public abstract class WorldFactory {
	
	private static final int WORLD_SIZE = 5000;
	private static final int NUM_START_FOOD = CreativeBehaviour.crebeCfg.getInt("num_start_foods");
	private static final double POISON_RATIO = CreativeBehaviour.crebeCfg.getDouble("poison_ratio");
	
	public static World makeStandardWorld() {
		World w = new StandardWorld(1000, 1000);
		/* Do stuff */
		return w;
	}
	
	public static World makeFoodWorld() {
		FoodWorld w = new FoodWorld(WORLD_SIZE, WORLD_SIZE);
		
		for (int i = 0; i < NUM_START_FOOD; i++) {
			Food f = new Food();
			f.position.x = Utils.randomFloat() * w.WIDTH;
			f.position.y = Utils.randomFloat() * w.HEIGHT;
			w.addFood(f);
		}
		
		return w;
	}
	
	public static World makeFoodWorldEdgeSpawn() {
		FoodWorld w = new FoodWorldEdgeSpawn(WORLD_SIZE, WORLD_SIZE);
		int foodsToSpawn = (int)(NUM_START_FOOD * FoodWorldEdgeSpawn.EDGE_SPAWN_RANGE);
		Pointf origin = new Pointf(WORLD_SIZE*0.5, WORLD_SIZE*0.5);
		
		for (int i = 0; i < foodsToSpawn;) {
			double x = Utils.randomDouble() * w.WIDTH;
			double y = Utils.randomDouble() * w.HEIGHT;
			Pointf spawn = new Pointf(x, y);
			
			if (spawn.distanceTo(origin) > WORLD_SIZE * (1-FoodWorldEdgeSpawn.EDGE_SPAWN_RANGE)) {
				Food f = new Food();
				f.position.x = x;
				f.position.y = y;
				w.addFood(f);
				i++;
			}	
		}
		
		return w;
	}
	
	public static World makeFoodAndPoisonWorld() {
		FoodWorld w = new FoodWorld(WORLD_SIZE, WORLD_SIZE);
		
		int numPoison = (int)(NUM_START_FOOD * POISON_RATIO);
		int numFoods = NUM_START_FOOD - numPoison;
		
		for (int i = 0; i < numFoods; i++) {
			Food f = new Food();
			f.position.x = Utils.randomFloat() * w.WIDTH;
			f.position.y = Utils.randomFloat() * w.HEIGHT;
			w.addFood(f);
		}
		
		for (int i = 0; i < numPoison; i++) {
			Food f = new Food(Food.DEFAULT_NOURISHMENT * -1);
			f.position.x = Utils.randomFloat() * w.WIDTH;
			f.position.y = Utils.randomFloat() * w.HEIGHT;
			w.addFood(f);
		}
		
		return w;
	}
	
	public static World makeArenaTrainingWorld(WorldType type) {
		World w = new ArenaWorld(800, 800, type);
		
		return w;
	}
	
	public static World makePresentationWorld() {
		return new PresentationWorld();
	}
}
