package creativebehaviour.util;

import java.awt.geom.Point2D;
import java.io.Serializable;
import java.util.Locale;


/**
 * A point class using double floating point precision.
 */
public class Pointf extends Point2D implements Serializable {

	public double x;
	public double y;

	
	/**
	 * Constructor.
	 * @param x
	 * @param y 
	 */
	public Pointf(double x, double y) {
		this.x = x;
		this.y = y;
	}

	
	/**
	 * Default constructor that initializes to (0, 0)
	 */
	public Pointf() {
		this(0, 0);
	}
	
	
	/**
	 * Copy another Pointf.
	 * @param orig 
	 */
	public Pointf(Pointf orig) {
		this(orig.x, orig.y);
	}
	
	
	/**
	 * Set this point equal to another point. The supplied point is not modified.
	 * @param other 
	 */
	public void setEqual(Pointf other) {
		x = other.x;
		y = other.y;
	}

	
	@Override
	public String toString() {
		return String.format(Locale.ENGLISH, "(%.2f, %.2f) len=%.2f", x, y, length());
	}
	
	
	public Pointf add(Pointf p) {
		this.x += p.x;
		this.y += p.y;
		
		return this;
	}
	
	public double distanceTo(Pointf p) {
		double dX = Math.abs(this.x - p.x);
		double dY = Math.abs(this.y - p.y);
		return Math.sqrt(dX*dX + dY*dY);
	}

	
	@Override
	public double getX() {
		return x;
	}

	
	@Override
	public double getY() {
		return y;
	}

	
	@Override
	public void setLocation(double x, double y) {
		this.x = x;
		this.y = y;
	}
	
	
	public Pointf scale(double s) {
		x *= s;
		y *= s;
		return this;
	}
	
	
	public double length() {
		return Math.sqrt(x*x + y*y);
	}
	
	
	public void normalize() {
		double len = length();
		x /= len;
		y /= len;
	}
	
}
