package creativebehaviour.util;

import java.io.File;
import java.util.Locale;
import java.util.Random;
import java.util.concurrent.ThreadLocalRandom;


public class Utils {
	
	private static final Random SEED_RNG = new Random();
	private static Random rng = new Random(0);
	
	
	public static void seed(long seed) {
		rng = new Random(seed);
	}
	
	
	public static long generateSeed() {
		return SEED_RNG.nextLong();
	}
	
	
	/**
	 * Seed random.
	 */
	public static void seed() {
		seedRandom();
	}
	
	
	public static void seedRandom() {
		rng = new Random();
	}
	
	
	/**
	 * Returns a pseudorandom double value between 0.0 and 1.0.
	 * @return 
	 */
	public static double randomDouble() {
		return rng.nextDouble();
		//return ThreadLocalRandom.current().nextDouble();
	}
	
	
	public static double randomDouble(double min, double max) {
		return rng.nextDouble() * (max - min) + min;
		//return ThreadLocalRandom.current().nextDouble(min, max);
	}
	
	
	/**
	 * Returns a pseudorandom float value between 0.0 and 1.0.
	 * @return 
	 */
	public static float randomFloat() {
		return rng.nextFloat();
		//return ThreadLocalRandom.current().nextFloat();
	}
	
	
	public static float randomFloat(float min, float max) {
		return rng.nextFloat() * (max - min) + min;
		//return randomFloat() * (max - min) + min;
	}
	
	
	/**
	 * Returns a pseudorandom integer between min (inclusive) and max (exclusive).
	 * @param min
	 * @param max
	 * @return 
	 */
	public static int randomInt(int min, int max) {
		return rng.nextInt(max - min) + min;
		//return ThreadLocalRandom.current().nextInt(min, max);
	}
	
	
	/**
	 * Returns a pseudorandom integer between 0 (inclusive) and max (exclusive).
	 * @param max
	 * @return 
	 */
	public static int randomInt(int max) {
		return rng.nextInt(max);
		//return ThreadLocalRandom.current().nextInt(max);
	}
	
	
	public static int randomInt() {
		return rng.nextInt();
		//return ThreadLocalRandom.current().nextInt();
	}
	
	
	public static long randomLong() {
		return rng.nextLong();
		//return ThreadLocalRandom.current().nextLong();
	}
	
	
	public static boolean randomBoolean() {
		return rng.nextBoolean();
		//return ThreadLocalRandom.current().nextBoolean();
	}
	
	
	public static boolean randomEvent(double prob) {
		return randomDouble() <= prob;
		//return randomDouble() <= prob;
	}
	
	
	public static double randomGausian(double mean, double std) {
		return rng.nextGaussian() * std + mean;
		//return ThreadLocalRandom.current().nextGaussian() * std + mean;
	}
	
	
	public static String getFileExtension(File f) {
		String ext = null;
		String s = f.getName();
		int i = s.lastIndexOf('.');

		if (i > 0 && i < s.length() - 1) {
			ext = s.substring(i+1).toLowerCase();
		}
		return ext;
	}
	
	
	/**
	 * Returns the smallest number in a double array.
	 * @param a
	 * @return 
	 */
	public static double min(double[] a) {
		int lowest = 0;
		for (int i = 1; i < a.length; i++) {
			if (a[i] < a[lowest]) lowest = i;
		}
		return a[lowest];
	}
	
	
	/**
	 * Returns the average of all the numbers in a double array.
	 * @param a
	 * @return 
	 */
	public static double avg(double[] a) {
		double sum = 0.0;
		for (int i = 0; i < a.length; i++) {
			sum += a[i];
		}
		return sum / (double) a.length;
	}
	
	
	public static String toString(double[] a) {
		StringBuilder str = new StringBuilder();
		for (int i = 0; i < a.length; i++) {
			str.append(String.format(Locale.ENGLISH, ", %6.3f", a[i]));
		}
		return "[" + str.substring(2) + "]";
	}
	
	
	public static String toString(float[] a) {
		StringBuilder str = new StringBuilder();
		for (int i = 0; i < a.length; i++) {
			str.append(String.format(Locale.ENGLISH, ", %6.3f", a[i]));
		}
		return "[" + str.substring(2) + "]";
	}
	
}

