package creativebehaviour.util.io;

import creativebehaviour.CreativeBehaviour;
//import creativebehaviour.graphics.evolvedlistgui.GenotypeWrapper;
import creativebehaviour.models.Creature;
import creativebehaviour.util.Utils;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class CrebeIO {
	public static final String FILE_EXT = "scd"; // Serialized Creature Dump
	public static final String EXPORT_PATH = CreativeBehaviour.crebeCfg.get("export_dir");
	
	
	static {
		File exportDir = new File(EXPORT_PATH);
		try {
			if (!exportDir.exists()) {
				exportDir.mkdirs();
			} else if (!exportDir.isDirectory()) {
				throw new IOException("Invalid creature export path \"" + exportDir.getAbsolutePath() +
						"\". File already exists. Change export directory in " + CreativeBehaviour.crebeCfg.filename);
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	
	/**
	 * Export a creature into the export directory.
	 * @param c
	 * @param filename
	 * @throws IOException 
	 */
	public static void exportCreature(Creature c, String filename) throws IOException {
		exportCreature(c, new File(EXPORT_PATH, filename));
	}
	
	
	/**
	 * Export a creature.
	 * @param c
	 * @param f
	 * @throws IOException 
	 */
	public static void exportCreature(Creature c, File f) throws IOException {
		// Ensure correct file extension
		if (!FILE_EXT.equalsIgnoreCase(Utils.getFileExtension(f))) {
			f = new File(f.getAbsolutePath() + "." + FILE_EXT);
		}
		
		try (ObjectOutputStream oos = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(f)))) {
			oos.writeObject(c);
			oos.flush();
		}
	}
	
	
	/**
	 * Import a creature stored in the export folder.
	 * @param filename
	 * @return 
	 * @throws java.io.IOException 
	 * @throws java.lang.ClassNotFoundException 
	 */
	public static Creature importCreature(String filename) throws IOException, ClassNotFoundException, ClassCastException {
		File f = new File(EXPORT_PATH, filename);
		return importCreature(f);
	}
	
	
	/**
	 * 
	 * @param f
	 * @return
	 * @throws IOException
	 * @throws ClassNotFoundException
	 * @throws ClassCastException 
	 */
	public static Creature importCreature(File f) throws IOException, ClassNotFoundException, ClassCastException {
		Object obj;
		try (ObjectInputStream ois = new ObjectInputStream(new BufferedInputStream(new FileInputStream(f)))) {
			obj = ois.readObject();
		}
		
		Creature c = (Creature) obj;
		return c;
	}
	
	
	/**
	 * Returns a list of all exported creatures in the export folder.
	 * @return 
	 */
	public static List<File> listExportedCreatures() {
		File exFolder = new File(EXPORT_PATH);
		List<File> list = new ArrayList<>();
		
		for (File file : exFolder.listFiles()) {
			if (file.isFile() && FILE_EXT.equalsIgnoreCase(Utils.getFileExtension(file))) {
				list.add(file);
			}
		}
		
		return list;
	}
	
	
	/**
	 * Deprecated.
	 * @param c
	 * @param f
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @deprecated
	 */
//	@Deprecated
//	public static void exportCreature_Old(ANNCreature c, File f) throws FileNotFoundException, IOException {
//		if (!GenotypeWrapper.FILE_EXT.equalsIgnoreCase(Utils.getFileExtension(f))) {
//			f = new File(f.getAbsolutePath() + "." + GenotypeWrapper.FILE_EXT);
//		}
//		
//		try (ObjectOutputStream oos = new ObjectOutputStream(new BufferedOutputStream(new FileOutputStream(f)))) {
//			GenotypeWrapper gw = c.createGenotypeWrapper();
//			oos.writeObject(gw);
//			oos.flush();
//		}
//	}
	
	
	/**
	 * Deprecated.
	 * @param f
	 * @return
	 * @throws IOException
	 * @throws ClassNotFoundException
	 * @deprecated
	 */
//	@Deprecated
//	public static ANNCreature importCreature_Old(File f) throws IOException, ClassNotFoundException {
//		Object obj;
//		try (ObjectInputStream ois = new ObjectInputStream(new BufferedInputStream(new FileInputStream(f)))) {
//			obj = ois.readObject();
//		}
//
//		GenotypeWrapper gw = (GenotypeWrapper)obj;
//		ANNCreature c = new ANNCreature(gw);
//		c.setFitness(gw.fitness);
//		
//		System.out.println("Imported creature: fitness=" + gw.fitness + ", ann config=" + Arrays.toString(gw.annHiddenConfig));
//		
//		return c;
//	}
	
}
