package creativebehaviour.util.profiling;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Timer {
	
	public static final int NANO = 1;
	public static final int MICRO = 1_000;
	public static final int MILLI = 1_000_000;
	public static final int SECOND = 1_000_000_000;
	
	private boolean paused;
	private long start_t;
	private long paused_et;
	private long paused_start_t;
	private int mode = NANO;
	
	
	public Timer(int mode) {
		this.mode = mode;
		paused_et = 0;
		start_t = System.nanoTime();
	}

	
	public Timer() {
		this(MILLI);
	}
	
	
	public void pause() {
		if (!paused) {
			paused_start_t = System.nanoTime();
			paused = true;
		}
	}
	
	
	public void resume() {
		if (paused) {
			paused_et += System.nanoTime() - paused_start_t;
			paused = false;
		}
	}
	
	
	public void reset() {
		start_t = System.nanoTime();
		if (paused) paused_start_t = start_t;
		paused_et = 0;
	}
	
	
	public boolean isPaused() {
		return paused;
	}
	
	
	public long getElapsed() {
		return toMode(System.nanoTime() - start_t - paused_et);
	}
	
	
	private long toMode(long dt) {
		return dt / mode;
	}
	
	
	private String getModeStr() {
		switch (mode) {
			case NANO: return "ns";
			case MICRO: return "µs";
			case MILLI: return "ms";
			case SECOND: return "s";
			default: throw new RuntimeException("Unknown timer mode: " + mode);
		}
	}

	
	@Override
	public String toString() {
		long et = getElapsed();
		return String.format("ET: %d %s", et, getModeStr());
	}
	
	
	
}
