package creativebehaviour.util.tree;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;


public class Node<T extends TreeElement<T> & Serializable> implements Iterable<Node<T>>, Serializable {

	public Node<T> parent;
	public ArrayList<Node<T>> children;
	public T obj;

	
	public Node(Node<T> parent, T obj) {
		this.parent = parent;
		this.children = new ArrayList<>();
		this.obj = obj;
	}
	
	
	public Node(T obj) {
		this(null, obj);
	}
	
	
	public Node() {
		this(null, null);
	}
	
	
	/**
	 * Copy constructor.
	 * @param orig
	 * @param ignored This argument is used to differentiate the different constructors. Its value is ignored.
	 */
	public Node(Node<T> orig, boolean ignored) {
		children = new ArrayList<>(orig.children.size());
		if (orig.obj != null) obj = orig.obj.deepCopy();
		
		for (Node<T> child : orig.children) {
			Node<T> childCpy = new Node<>(child, false);
			childCpy.parent = this;
			children.add(childCpy);
		}
	}
	
	
	public int getNumChildren() {
		return children.size();
	}
	
	
	public void addChildNode(Node<T> child) {
		children.add(child);
		child.parent = this;
	}
	
	
	public void addChildObj(T obj) {
		addChildNode(new Node<>(obj));
	}
	
	
	public void setChildNode(Node<T> child, int index) {
		children.set(index, child);
		child.parent = this;
	}
	
	
	public Node<T> getChild(int index) {
		return children.get(index);
	}
	
	
	/**
	 * Use copy constructor instead.
	 * @return
	 * @deprecated
	 */
	@Deprecated
	public Node<T> clone() {
		Node<T> n = new Node<>();
		
		for (Node<T> child : children) {
			n.addChildNode(child.clone());
		}
		
		return n;
	}

	
	@Override
	public Iterator<Node<T>> iterator() {
		return children.iterator();
	}
	

	public int getNumElements() {
		int count = obj == null ? 0 : 1;
		for (Node<T> child : children) {
			count += child.getNumElements();
		}
		return count;
	}

	
	@Override
	public String toString() {
		return String.format("Node<%s>", obj == null ? "null" : obj.getClass().getSimpleName());
	}
	
	
	
	
	
}
