package creativebehaviour.util.tree;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Stack;


public class Tree<T extends TreeElement<T> & Serializable> implements Iterable<T>, Serializable {

	public Node<T> root;

	
	public Tree() {
		root = new Node<>();
	}
	
	
	/**
	 * Copy constructor. Creates a deep copy of a tree, retaining references to
	 * the original objectes stored in the tree. The data objects are not copied.
	 * @param orig 
	 */
	public Tree(Tree<T> orig) {
		root = new Node<>(orig.root, false);
	}
	
	
	/**
	 * Use copy constructor instead.
	 * @return
	 * @deprecated
	 */
	@Deprecated
	public Tree<T> clone() {
		Tree<T> t = new Tree<>();
		
		t.root = this.root.clone();
		
		return t;
	}
	
	
	/**
	 * Calculates the number of elements stored in this tree.
	 * @return 
	 */
	public int getNumElements() {
		return root.getNumElements();
	}
	

	/**
	 * Iterator for every non-null element stored in this tree.
	 * @return 
	 */
	@Override
	public Iterator<T> iterator() {
		return new TreeIterator();
	}
	
	
	/**
	 * Iterator for every node in this tree.
	 * @return 
	 */
	public Iterable<Node<T>> nodeIterator() {
		return new NodeIterator();
	}
	
	
	
	private class TreeIterator implements Iterator<T> {
		
		private int count = 0;
		private final ArrayList<T> list = new ArrayList<>(25);
		
		public TreeIterator() {
			Stack<Node<T>> stack = new Stack<>();
			stack.push(root);
			Node<T> currentNode;
			
			while (!stack.isEmpty()) {
				currentNode = stack.pop();
				if (currentNode.obj != null) list.add(currentNode.obj);
				stack.addAll(currentNode.children);
			}
		}
		
		
		@Override
		public boolean hasNext() {
			return count < list.size();
		}

		
		@Override
		public T next() {
			return list.get(count++);
		}
		
	}
	
	
	
	private class NodeIterator implements Iterable<Node<T>>, Iterator<Node<T>> {

		private int count = 0;
		private final ArrayList<Node<T>> list = new ArrayList<>(25);

		public NodeIterator() {
			Stack<Node<T>> stack = new Stack<>();
			stack.push(root);
			Node<T> currentNode;
			
			while (!stack.isEmpty()) {
				currentNode = stack.pop();
				list.add(currentNode);
				stack.addAll(currentNode.children);
			}
		}
		
		
		@Override
		public boolean hasNext() {
			return count < list.size();
		}

		
		@Override
		public Node<T> next() {
			return list.get(count++);
		}

		
		@Override
		public Iterator<Node<T>> iterator() {
			return this;
		}
		
	}
}
