package eval.ea.selection.adult;

import eval.models.Population;
import java.io.Serializable;
import eval.util.Config;



public abstract class AdultSelection implements Serializable {
	
	/**
	 * Select and put n offspring into population. This will remove any existing elements in population.
	 * @param offspring
	 * @param population
	 * @param n 
	 */
	public void selectAdults(Population offspring, Population population, int n) {
		selectAdults(offspring, population, n, true);
	}
	
	public abstract void selectAdults(Population offspring, Population population, int n, boolean maximize);
	
	
	public static AdultSelection parseSelection(Config cfg) {
		String name = cfg.get("adult_selection");
		String type = name.toLowerCase().replaceAll("(_*|\\s*)", "");
		
		if (type.startsWith("generationalreplacement")) {
			return new GenerationalReplacementAdultSelection();
		}
		
		if (type.startsWith("generationalmixing")) {
			return new GenerationalMixingAdultSelection();
		}
		
		if (type.startsWith("overproduction")) {
			return new OverproductionAdultSelection();
		}
		
		throw new RuntimeException("Missing adult selection algorithm: "+name);
	}
}
