package eval.ea.selection.parent;

import eval.models.Phenotype;
import eval.models.Population;
import java.io.Serializable;
import java.util.List;
import eval.util.Config;
import eval.util.Pair;

public abstract class ParentSelection implements Serializable {
	
	/**
	 * Select n pairs (2n parents in total) from p.
	 * @param p
	 * @param n
	 * @return 
	 */
//	public List<Pair<Phenotype>> selectParents(Population p, int n) {
//		return selectParents(p, n, true);
//	}
	
	public List<Pair<Phenotype>> selectParents(Population p, int n) {
		return selectParents(p, n, true);
	}
	
	public abstract List<Pair<Phenotype>> selectParents(Population p, int n, boolean maximize);
	
	
	public static ParentSelection parseSelection(Config cfg) {
		String name = cfg.get("parent_selection");
		String type = name.toLowerCase().replaceAll("(_*|\\s*)", "");
		
		if (type.startsWith("tournament")) {
			int size = cfg.getInt("tournament_size");
			double prob = cfg.getDouble("tournament_prob");
			return new TournamentParentSelection(size, prob);
		}
		
		if (type.startsWith("sigmascaling")) {
			return new SigmaScalingParentSelection();
		}
		
		if (type.startsWith("proportionate")) {
			return new ProportionateParentSelection();
		}
		
		if (type.startsWith("rank")) {
			return new RankParentSelection();
		}
		
		throw new RuntimeException("Missing parent selection algorithm: "+name);
	}
}
