package eval.ea.selection.parent;

import eval.models.Phenotype;
import eval.models.Population;
import java.util.ArrayList;
import java.util.List;
import eval.util.Pair;
import eval.util.roulette.Roulette;


public class RankParentSelection extends ParentSelection {
	
	private static final double MIN = 0.5;
	private static final double MAX = 1.5;

	@Override
	public List<Pair<Phenotype>> selectParents(Population p, int n, boolean maximize) {
		if (!maximize)
			throw new UnsupportedOperationException("Minimization in RankParentSelection is not implemented.");
		
		ArrayList<Pair<Phenotype>> parents = new ArrayList<>(n);
		Population sortedPop = p.getSorted(false);
		Roulette<Phenotype> rw = new Roulette<>(n);
		
		for (int i = 0; i < n; i++) {
			rw.add(sortedPop.getPhenotype(i), MIN + (MAX - MIN) * (i - 1) / (double) (n - 1));
		}
		
		for (int i = 0; i < n; i++) {
			Pair<Phenotype> pair = new Pair<>(rw.spin(false), rw.spin(false));
			parents.add(pair);
		}
		
		return parents;
	}

}
