

package eval.ea.selection.parent;

import eval.models.Phenotype;
import eval.models.Population;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import eval.util.Pair;
import eval.util.Utils;

/**
 *
 * @author Endre
 */
public class TournamentParentSelection extends ParentSelection {
	
	private final int tournamentSize;
	private final double prob;
	
	
	public TournamentParentSelection(int tournamentSize, double prob) {
		this.tournamentSize = tournamentSize;
		this.prob = prob;
	}
	
	
	@Override
	public List<Pair<Phenotype>> selectParents(Population p, int n, boolean maximize) {
		List<Pair<Phenotype>> parents = new ArrayList<>(n);
		
		while (parents.size() < n) {
			ArrayList<Phenotype> pool = makePool(p, tournamentSize, maximize);
			parents.add(selectWinners(pool));
		}
		
		return parents;
	}
	
	
	/**
	 * Returns a tournament pool containing phenotypes where 
	 * each pool is sorted to have the highest fitness first
	 * @param p
	 * @return 
	 */
	private ArrayList<Phenotype> makePool(Population p, int size, boolean maximize) {
		ArrayList<Phenotype> pool = new ArrayList<>(size);
		ArrayList<Phenotype> pop = p.getPopClone();
		
		while (pool.size() < size){
			pool.add(pop.remove(Utils.randomInt(pop.size())));
		}
		
		if (maximize)
			Collections.sort(pool, Comparator.reverseOrder());
		else
			Collections.sort(pool);
		
		return pool;
	}
	
	
	/**
	 * Returns a pair that won the tournament
	 * @param pool
	 * @return 
	 */
	private Pair<Phenotype> selectWinners(ArrayList<Phenotype> pool) {
		// Selecting parent 1
		Phenotype parent1;
		if (Utils.randomDouble() < prob) parent1 = pool.get(0); // Gets the best
		else parent1 = pool.get(Utils.randomInt(pool.size()));  // Gets random
		
		// Selecting parent 2
		Phenotype parent2;
		if (Utils.randomDouble() < prob) parent2 = pool.get(1); // Gets the next best
		else parent2 = pool.get(Utils.randomInt(pool.size()));  // Gets random
		
		return new Pair<>(parent1, parent2);
	}
	
}
