package eval.example;

import eval.models.BinaryGenotype;
import eval.models.Genotype;
import eval.models.Phenotype;
import eval.util.Utils;



public class LOLZPhenotype extends Phenotype {
	
	private final int            Z;
	private final BinaryGenotype geno;
	private int                  lolzcount;

	
	public LOLZPhenotype(Genotype geno, int z) {
		super(geno);
		this.geno = (BinaryGenotype)geno;
		this.Z = z;
	}

	
	@Override
	public Genotype getGenotype() {
		return genotype;
	}

	
	@Override
	public double calculateFitness() {
		boolean prefix = geno.bits[0];
		int lolz = 0;
		
		for (int i = 0; i < geno.bits.length; i++) {
			if (geno.bits[i] != prefix) 
				break;
			lolz++;
		}
		
		if (!prefix && lolz > Z) 
			lolz = Z;
		lolzcount = lolz;
		
		return lolz / (double)(geno.bits.length);
	}

	
	@Override
	public String toString() {
		StringBuilder gstr = new StringBuilder();
		for (int i = 0; i < geno.bits.length; i++) {
			gstr.append(geno.bits[i] ? '|' : '_');
		}
		return "LOLZPhenotype{values=" + gstr + ", lolz=" + lolzcount +'}';
	}
}
