package eval.models;

import eval.util.Utils;


public class BinaryGenotype extends Genotype{
	
	public boolean[] bits;
	
	public BinaryGenotype(boolean... bits){
		this.bits = bits;
	}
	
	public BinaryGenotype(int bitcount){
		this.bits = new boolean[bitcount];
	}
	

	@Override
	public Genotype copy() {
		boolean[] clonedBits = new boolean[this.bits.length];
		System.arraycopy(this.bits, 0, clonedBits, 0, this.bits.length);
		return new BinaryGenotype(clonedBits);
	}

	
	@Override
	public Genotype crossover(Genotype g, Crossover type) {
		switch(type){	
			case ONE_POINT: return onePointCrossover((BinaryGenotype)g);
			default:
				throw new AssertionError(type.name());
		}	
	}
	
	
	private BinaryGenotype onePointCrossover(BinaryGenotype g){
		int splitIndex = Utils.randomInt(this.bits.length);
		boolean[] bits = new boolean[this.bits.length];
		System.arraycopy(this.bits, 0, bits, 0, splitIndex);
		System.arraycopy(g.bits, splitIndex, bits, splitIndex, bits.length - splitIndex);
		return new BinaryGenotype(bits);
	}
	

	@Override
	public void mutate(double prob) {
		for (int i = 0; i < this.bits.length; i++){
			if (Utils.randomDouble() < prob) {
				this.bits[i] = !this.bits[i];
			}
		}
	}

	
	@Override
	public String toString() {
		StringBuilder gstr = new StringBuilder();
		for (int i = 0; i < bits.length; i++) {
			gstr.append(bits[i] ? '|' : '_');
		}
		return "BinaryGenotype{" + gstr + '}';
	}
}
