package eval.models;

import eval.util.roulette.RouletteEntry;
import java.io.Serializable;


public abstract class Phenotype implements Comparable<Phenotype>, RouletteEntry, Serializable {
	
	protected boolean evaluated;
	protected double fitness;
	protected Genotype genotype;


	public Phenotype(Genotype g) {
		this.evaluated = false;
		this.fitness = 0.0;
		this.genotype = g;
	}

	
	@Override
	public int compareTo(Phenotype o) {
		if (fitness > o.fitness)
			return 1;
		if (fitness < o.fitness)
			return -1;
		return 0;
	}
	
	
	public double getFitness() {
		return fitness;
	}

	@Override
	public double getRouletteValue() {
		return fitness;
	}
	
	public Genotype getGenotype() {
		return genotype;
	}
	
	/**
	 * Returns true if this phenotype has been fitness tested.
	 * @return 
	 */
	public boolean isEvaluated() {
		return evaluated;
	}
	
	/**
	 * Calculate the fitness of this phenotype.
	 */
	public void evaluate() {
		fitness = calculateFitness();
		evaluated = true;
	}
	
	/**
	 * This should do the heavy stuff and calculate the fitness value, 
	 * which the other functions can use by calling get methods
	 * @return 
	 */
	protected abstract double calculateFitness();
}
