package eval.util;

import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Collection;
import java.util.Properties;
import java.util.Set;



public class Config implements Serializable {
	
	private final Properties prop;
	public final String filename;

	public Config(String filename) throws IOException {
		this.filename = filename;
		prop = new Properties();
		prop.load(new FileInputStream(filename));
	}
	
	
	/**
	 * Write config to disk.
	 * @throws java.io.FileNotFoundException
	 */
	public void store() throws FileNotFoundException, IOException {
		try (FileOutputStream fos = new FileOutputStream(filename, false)) {
			prop.store(fos, null);
		}
	}
	
	
	public String get(String key) {
		if (!prop.containsKey(key)) {
			throw new RuntimeException("Missing key: "+key);
		}
		return prop.getProperty(key);
	}
	
	/**
	 * Set a value in the config.
	 * @param key
	 * @param value 
	 */
	public void set(String key, String value) {
		prop.setProperty(key, value);
	}
	
	public int getInt(String key) {
		return Integer.parseInt(get(key));
	}
	
	public int getInt(String key, int defaultVal) {
		return prop.containsKey(key) ? getInt(key) : defaultVal;
	}
	
	public long getLong(String key) {
		return Long.parseLong(get(key));
	}
	
	public long getLong(String key, long defaultVal) {
		return prop.containsKey(key) ? getLong(key) : defaultVal;
	}
	
	public double getDouble(String key) {
		return Double.parseDouble(get(key));
	}
	
	public double getDouble(String key, double defaultVal) {
		return prop.containsKey(key) ? getDouble(key) : defaultVal;
	}
	
	public float getFloat(String key) {
		return Float.parseFloat(get(key));
	}
	
	public float getFloat(String key, float defaultVal) {
		return prop.containsKey(key) ? getFloat(key) : defaultVal;
	}
	
	public boolean getBoolean(String key) {
		return Boolean.parseBoolean(get(key));
	}
	
	public boolean getBoolean(String key, boolean defaultVal) {
		return prop.containsKey(key) ? getBoolean(key) : defaultVal;
	}
	
	public Set<Object> keySet() {
		return prop.keySet();
	}
	
	public Collection<Object> values() {
		return prop.values();
	}
	
	public boolean has(String key) {
		return prop.containsKey(key);
	}
}
