package eval.util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;



public class Logger {
	
	public static boolean log = true;
	public static boolean logtostdout = true;
	public static boolean logtofile = false;
	public static boolean usecolors = true;
	public static final File logfile;
	public static final File rootdir = new File(".");
	public static final Object writelock = new Object();
	public static FileWriter writer;
	
	public static final SimpleDateFormat dateformatter = new SimpleDateFormat("yyyy.MM.dd HH:mm:ss:SSS");
	public static final SimpleDateFormat compactdateformatter = new SimpleDateFormat("yyyyMMdd_HH-mm-ss");
	
	public static final String red     = "[1;31m";
	public static final String green   = "[1;32m";
	public static final String yellow  = "[1;33m";
	public static final String blue    = "[1;34m";
	public static final String magenta = "[1;35m";
	public static final String cyan    = "[1;36m";
	
	
	static {
		String filename = compactdateformatter.format(new Date())+".log";
		logfile = new File(rootdir, filename);
		try {
			if (logtofile)
				writer = new FileWriter(logfile, true);
		} catch (IOException e) {
			writer = null;
		}
	}
	
	
	public static void info(String msg) {
		String tag = "INFO";
		print(yellow,tag, msg);
		writeToFile(tag, msg);
	}
	
	public static void warning(String msg) {
		String tag = "WARN";
		print(magenta, tag, msg);
		writeToFile(tag, msg);
	}
	
	public static void debug(Object obj) {
		debug(obj.toString());
	}
	
	public static void debug(String msg) {
		String tag = "DEBUG";
		print(cyan, tag, msg);
		writeToFile(tag, msg);
	}
	
	public static void error(String msg) {
		String tag = "ERROR";
		print(red, tag, msg);
		writeToFile(tag, msg);
	}
	
	public static void notify(String msg) {
		String tag = "NOTE";
		print(cyan, tag, msg);
		writeToFile(tag, msg);
	}
	
	public static void msg(String msg) {
		String tag = null;
		print(null, tag, msg);
		writeToFile(tag, msg);
	}
	

	public static void print(String colorstr, String tag, String msg) {
		if (!log || !logtostdout)
			return;
		
		String caller = getCallerCallerClassName();
		if (tag != null)
			tag += " "+caller.substring(caller.lastIndexOf('.')+1);
		else
			tag = caller.substring(caller.lastIndexOf('.')+1);
		synchronized (System.out) {
			if (colorstr != null && usecolors) {
				System.out.print((char)0x1b);
				System.out.print(colorstr);
			}
			if (tag != null) {
				System.out.printf("%-9s","["+tag+"] ");
			}
			System.out.print(dateformatter.format(new Date()));
			System.out.print(" - ");
			System.out.print(msg);
			if (colorstr != null && usecolors) {
				System.out.print((char)0x1b);
				System.out.print("[0m");
			}
			System.out.println();
		}
	}
	
	
	public static void writeToFile(String tag, String msg) {
		if (!log || !logtofile)
			return;
		synchronized (writelock) {
			try {
				if (tag != null) {
					writer.write(String.format("%-9s","["+tag+"] "));
				}
				writer.write(dateformatter.format(new Date()));
				writer.write(" - ");
				writer.write(msg);
				writer.write(System.lineSeparator());
				writer.flush();
			} catch (IOException e) {
				System.err.println("LOGGING ERROR. Cannot write to file: "+e.getMessage());
			}
		}
	}
	
	public static String getCallerCallerClassName() {
		StackTraceElement[] stElements = Thread.currentThread().getStackTrace();
		return stElements[4].getClassName()+"::"+stElements[4].getMethodName();
	}

	public static void main(String[] args) {
		Logger.msg("TEST");
		Logger.debug("TEST");
		Logger.info("TEST");
		Logger.notify("TEST");
		Logger.warning("TEST");
		Logger.error("TEST");
	}
}	
