package eval.util;

import java.util.Random;
import java.util.concurrent.ThreadLocalRandom;

/**
 *
 * @author Endre
 */
public class Utils {
	
	private static Random rng = new Random();
	
	public static void seed(long seed) {
		rng = new Random(seed);
	}
	
	public static void seed() {
		rng = new Random();
	}
	
	// Should use ThreadLocalRandom for concurrent operation, but Random is thread safe so is ok.
	
	/**
	 * Returns a pseudorandom double value between 0.0 and 1.0.
	 * @return 
	 */
	public static double randomDouble() {
		return rng.nextDouble();
		//return ThreadLocalRandom.current().nextDouble();
	}
	
	public static double randomDouble(double max) {
		return rng.nextDouble() * max;
	}
	
	public static double randomDouble(double min, double max) {
		return rng.nextDouble() * (max - min) + min;
	}
	
	public static float randomFloat() {
		return rng.nextFloat();
	}
	
	public static float randomFloat(float max) {
		return rng.nextFloat() * max;
	}
	
	public static float randomFloat(float min, float max) {
		return rng.nextFloat() * (max - min) + min;
	}
	
	/**
	 * Returns a pseudorandom integer between min (inclusive) and max (exclusive).
	 * @param min
	 * @param max
	 * @return 
	 */
	public static int randomInt(int min, int max) {
		return rng.nextInt(max - min) + min;
		//return ThreadLocalRandom.current().nextInt(min, max);
	}
	
	
	/**
	 * Returns a pseudorandom integer between 0 (inclusive) and max (exclusive).
	 * @param max
	 * @return 
	 */
	public static int randomInt(int max) {
		return rng.nextInt(max);
		//return ThreadLocalRandom.current().nextInt(max);
	}
	
	
	public static int randomInt() {
		return rng.nextInt();
		//return ThreadLocalRandom.current().nextInt();
	}
	
	
	public static long randomLong() {
		return rng.nextLong();
		//return ThreadLocalRandom.current().nextLong();
	}
	
	
	public static boolean randomEvent(double prob) {
		return randomDouble() <= prob;
		//return randomDouble() <= prob;
	}
	
	public static boolean randomEvent() {
		return rng.nextBoolean();
	}
	
	
	private static int last = -1;
	public static void progressbar(int length, double current, double max) {
		double percent = current / max;
		int intpercent = (int)(percent * 100);
		if (intpercent == last) {
			return;
		}
		last = intpercent;
		System.out.print("\r[");
		int pos = (int)(length * percent);
		for (int i = 0; i < length; i++) {
			if (i < pos)
				System.out.print("=");
			else
				System.out.print(" ");
		}
		System.out.print("] "+intpercent+"% ");
		System.out.flush();
	}
	
	

	public static StringBuilder buildProgressbar(StringBuilder sb, int length, double current, double max) {
		double percent = current / max;
		int intpercent = (int)(percent * 100);

		if (sb == null)
			sb = new StringBuilder();
		sb.append("[");
		int pos = (int)(length * percent);
		double part = (length * percent) - pos;
		boolean filled = false;
		for (int i = 0; i < length; i++) {
			if (i < pos) {
				sb.append("=");
			} else if (part > 0.5 && !filled) {
				sb.append("-");
				filled = true;
			} else {
				sb.append(" ");
			}
		}
		sb.append("] ");
		sb.append(String.format("%3d", intpercent));
		sb.append("% ");
		return sb;
	}
	
	
	public static StringBuilder buildShortProgressbar(StringBuilder sb, int length, double current, double max) {
		double percent = current / max;
		int intpercent = (int)(percent * 100);
		int pos = (int)(length * percent);
		double part = (length * percent) - pos;
		boolean filled = false;
		
		if (sb == null)
			sb = new StringBuilder();
		sb.append("[");
		for (int i = 0; i < length; i++) {
			if (i < pos) {
				sb.append("=");
			} else if (part > 0.5 && !filled) {
				sb.append("-");
				filled = true;
			} else {
				sb.append(" ");
			}
		}
		sb.append("]");
		return sb;
	}
	
	
	public static void slp(long ms) {
		try {
			Thread.sleep(ms);
		} catch (Exception e) {}
	}
}
