package creativebehaviour.fuzzy;

import java.util.ArrayList;
import java.util.List;

/**
 * Zero-order Sugeno set.
 * 
 * @author Bjørnar W. Alvestad
 */
public class SugenoSet extends Set {
	
	private final List<Double> values;
	private final List<String> labels;
	
	
	public SugenoSet(int var) {
		super(var);
		this.values = new ArrayList<>();
		this.labels = new ArrayList<>();
	}
	
	
	public SugenoSet(String varName) {
		this(Variable.indexOf(varName));
	}
	

	/**
	 * Copy constructor.
	 * @param original
	 */
	public SugenoSet(SugenoSet original) {
		super(original.var);
		this.values = new ArrayList<>(original.values);
		this.labels = new ArrayList<>(original.labels);
	}
	
	
	public void addValue(String name, double value) {
		labels.add(name);
		values.add(value);
	}

	@Override
	public String toString() {
		return "sugenoset:" + getName();
	}
	
	/**
	 * Calculates the weighted average.
	 * @param weights
	 * @return 
	 */
	public double eval(double[] weights) {
		if (weights.length != values.size())
			throw new IllegalArgumentException("Wrong number of arguments passed to eval. Expected " + values.size() + ", got " + weights.length + ".");
		
		double weightSum = 0.0;
		double sum = 0.0;
		for (int i = 0; i < weights.length; i++) {
			double weight = weights[i];
			weightSum += weight;
			sum += weight * values.get(i);
		}
		
		return weightSum < 1e-8 ? 0.0 : sum / weightSum;
	}
	
}
