package creativebehaviour.fuzzy.memberfunc;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Gaussian extends MembershipFunc {
	
	private double mean;
	private double variance;
	private double normalizationFactor;
	private double xScaleFactor;

	/**
	 * Gaussian membership function, normalized to one, meaning f(mean) == 1 in all cases.
	 * @param mean
	 * @param variance 
	 */
	Gaussian(double mean, double variance) {
		super(MembershipFunctions.GAUSSIAN);
		this.mean = mean;
		this.variance = variance;
		this.xScaleFactor = 1.0;
		
		normalizationFactor = 1 / gaussian(mean, mean, variance);
	}

	
	Gaussian(Gaussian original) {
		this(original.mean, original.variance);
		this.xScaleFactor = original.xScaleFactor;
	}
	
	
	@Override
	public double f(double x) {
		return normalizationFactor * gaussian(x * xScaleFactor, mean, variance);
	}
	

	@Override
	public String toString() {
		return super.toString() + "{" + "mean=" + mean + ", variance=" + variance + ", normalizationFactor=" + normalizationFactor + '}';
	}
	
	
	private static double gaussian(double x, double mean, double variance) {
		return (Math.exp(-(x-mean)*(x-mean)/(2*variance)))/Math.sqrt(2*variance*Math.PI);
	}
	

	@Override
	public void normalize(double normalizeFactor) {
		// this method has nothing to do with this.normalizationFactor, which scales in y dir.
		// this method scales in x dir.
		this.xScaleFactor = normalizeFactor;
	}

	
	@Override
	public MembershipFunc copy() {
		return new Gaussian(this);
	}
	
}
