package creativebehaviour.fuzzy.memberfunc;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public enum MembershipFunctions {
	GAUSSIAN      (false, 2),
	GRADE         (true,  2),
	REVERSE_GRADE (false, 2),
	POLYNOMIAL    (true,  2),
	SIGMOIDAL     (true,  2),
	TRAPEZOIDAL   (false, 4),
	TRIANGLE      (false, 3);
	
	/** Reversable flag. */
	public final boolean reversable;
	
	/** Number of parameters for this function, not including the reverse parameter. */
	public final int numParameters;
	
	
	private MembershipFunctions(boolean reversable, int numParameters) {
		this.reversable = reversable;
		this.numParameters = numParameters;
	}
	
	
	/**
	 * Create a new instance of a membership function.
	 * @param params
	 * @param reverse Ignored if this membership function is non-reversable.
	 * @return 
	 */
	public MembershipFunc createInstance(boolean reverse, double... params) {
		if (params.length != numParameters)
			throw new IllegalArgumentException("Wrong number of parameters to create " + name() + ". Need " + numParameters + ", got " + params.length + ".");
		
		switch (this) {
			case GAUSSIAN:
				return new Gaussian(params[0], params[1]);
			case GRADE:
				return new Grade(params, reverse);
			case REVERSE_GRADE:
				return new Grade(params, true);
			case POLYNOMIAL:
				return new Polynomial(params[0], params[1], reverse);
			case SIGMOIDAL:
				return new Sigmoidal(params[0], params[1], reverse);
			case TRAPEZOIDAL:
				return new Trapezoid(params);
			case TRIANGLE:
				return new Triangle(params);
				
			default:
				throw new AssertionError(this.name());
			
		}
	}
	
	
	/**
	 * Create a new instance of a membership function, always setting reversed to false.
	 * @param params
	 * @return 
	 */
	public MembershipFunc createInstance(double... params) {
		return createInstance(false, params);
	}
}
