package creativebehaviour.fuzzy.memberfunc;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Polynomial extends MembershipFunc implements Reversable {
	// http://img.yeskis.com/e38c6bc4c4b3ad99aa40044d05cfc4a3af271b27
	
	/** Left point. */
	private double a;
	
	/** Midpoint. */
	private double b;
	
	/** Right point. */
	private double y;
	
	private final double r;
	
	/** Flag to indicate reversed polynomial. */
	private final boolean reverse;

	
	Polynomial(double c, double r, boolean reverse) {
		super(MembershipFunctions.POLYNOMIAL);
		this.b = c;
		this.r = r;
		this.a = c - r;
		this.y = c + r;
		this.reverse = reverse;
	}

	
	Polynomial(Polynomial original) {
		super(MembershipFunctions.POLYNOMIAL);
		this.b = original.b;
		this.a = original.a;
		this.y = original.y;
		this.r = original.r;
		this.reverse = original.reverse;
	}
	
	
	@Override
	public double f(double x) {
		if (x <= a) x = 0;
		else if (x <= b) {
			double t = (x-a)/(y-a);
			x = 2*t*t;
		}
		else if (x < y) {
			double t = (x-y)/(y-a);
			x = 1-2*t*t;
		}
		else x = 1;
		return reverse ? 1 - x : x;
	}
	

	@Override
	public void normalize(double normalizeFactor) {
		a /= normalizeFactor;
		b /= normalizeFactor;
		y /= normalizeFactor;
	}

	
	@Override
	public MembershipFunc copy() {
		return new Polynomial(this);
	}

	
	@Override
	public MembershipFunc getReversed() {
		return new Polynomial(b, r, !reverse);
	}

}
