package creativebehaviour.fuzzy.memberfunc;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class Sigmoidal extends MembershipFunc implements Reversable {
	
	/** Steepness. */
	private double k;
	
	/** Centerpoint. */
	private double c;
	
	/** Flag to indicate reversed sigmoidal. */
	private final boolean reverse;

	
	Sigmoidal(double k, double c, boolean reverse) {
		super(MembershipFunctions.SIGMOIDAL);
		this.k = k;
		this.c = c;
		this.reverse = reverse;
	}
	

	Sigmoidal(Sigmoidal original) {
		super(MembershipFunctions.SIGMOIDAL);
		this.k = original.k;
		this.c = original.c;
		this.reverse = original.reverse;
	}
	
	
	@Override
	public double f(double x) {
		x = 1 / (1 + Math.exp(-k*(x-c)));
		return reverse ? 1 - x : x;
	}
	

	@Override
	public void normalize(double normalizeFactor) {
		c /= normalizeFactor;
		k *= normalizeFactor;
	}
	

	@Override
	public MembershipFunc copy() {
		return new Sigmoidal(this);
	}

	
	@Override
	public MembershipFunc getReversed() {
		return new Sigmoidal(k, c, !reverse);
	}
	

}
