package creativebehaviour.fuzzy.rule;

import creativebehaviour.fuzzy.Variable;
import java.io.Serializable;
import java.util.Objects;

/**
 *
 * @author Bjørnar W. Alvestad
 */
public class RuleClause implements Serializable {

	public final int var;
	public int value;
	public final boolean negation;
	public final int entitySensorIndex;

	
	public RuleClause(int var, int entitySensorIndex, int value, boolean negation) {
		this.var = var;
		this.value = value;
		this.negation = negation;
		this.entitySensorIndex = entitySensorIndex;
	}
	
	
	public RuleClause(int var, int varIndex, int value) {
		this(var, varIndex, value, false);
	}
	
	
	/**
	 * Copy constructor.
	 * @param original 
	 */
	public RuleClause(RuleClause original) {
		this(original.var, original.entitySensorIndex, original.value, original.negation);
	}
	
	
	/**
	 * Gets the corresponding singleton Variable object for this clause.
	 * @return 
	 */
	public Variable getVar() {
		return Variable.getVar(var);
	}
	
	
	@Override
	public String toString() {
		Variable var = getVar();
		return var.name + (entitySensorIndex == -1 ? "?" : entitySensorIndex) + (negation ? " IS NOT " : " IS ") + var.getValue(value).name;
	}

	
	@Override
	public int hashCode() {
		int hash = 7;
		hash = 47 * hash + Objects.hashCode(this.var);
		hash = 47 * hash + this.value;
		hash = 47 * hash + (this.negation ? 1 : 0);
		hash = 47 * hash + this.entitySensorIndex;
		return hash;
	}

	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final RuleClause other = (RuleClause) obj;
		if (this.value != other.value) {
			return false;
		}
		if (this.negation != other.negation) {
			return false;
		}
		if (this.entitySensorIndex != other.entitySensorIndex) {
			return false;
		}
		if (!Objects.equals(this.var, other.var)) {
			return false;
		}
		return true;
	}
	
	
	
}
